from nomad_media_cli.helpers.utils import initialize_sdk
import click
import sys

@click.command()
@click.pass_context
def clear_server_cache(ctx):
    """Clear the server cache"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        nomad_sdk.clear_server_cache()
        click.echo("Server cache cleared successfully")

    except Exception as e:
        click.echo(json.dumps({ "error": f"Error clearing server cache: {e}" }))
        sys.exit(1)


import click
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--live-channel-id", required=True, help="The ID of the live channel.")
@click.option("--wait-for-start", required=False, type=click.BOOL, help="Indicates if the live channel should wait for start.")
@click.pass_context
def start_live_channel(ctx, live_channel_id, wait_for_start):
    """Start live channel"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        nomad_sdk.start_live_channel(live_channel_id, wait_for_start)
        click.echo("Live channel started successfully.")

    except Exception as e:
        click.echo(json.dumps({"error": f"Error starting live channel: {e}"}))
        sys.exit(1)


import click
import json
import sys
from nomad_media_cli.helpers.utils import initialize_sdk
from nomad_media_cli.helpers.get_content_definition_id import get_content_definition_id
from nomad_media_cli.helpers.get_id_from_url import get_id_from_url

@click.command()
@click.option("--id", help="The ID of the content.")
@click.option("--url", help="The Nomad URL of the Asset (file or folder) to add the tag to (bucket::object-key).")
@click.option("--object-key", help="Object-key of the Asset (file or folder) to add the tag to. This option assumes the default bucket that was previously set with the `set-bucket` command.")
@click.option("--tag-name", help="The name of the tag.")
@click.option("--tag-id", help="The ID of the tag.")
@click.pass_context
def add_asset_tag(ctx, id, url, object_key, tag_name, tag_id):
    """Add tag to content"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]
    
    if not id and not url and not object_key:
        click.echo(json.dumps({"error": "Please provide either an id, url or object-key"}))
        sys.exit(1)

    if url or object_key:
        id = get_id_from_url(ctx, url, object_key, nomad_sdk)

    try:
        offset = 0
        tag_content_definition_id = get_content_definition_id(ctx, "Tag")
        while not tag_id:
            tags = nomad_sdk.search(None, offset, None, [{
                "fieldName": "contentDefinitionId",
                "operator": "equals",
                "values": tag_content_definition_id
            }], None, None, None, None, None, None, None, None, None, None, None)
            
            if len(tags["items"]) == 0:
                break
            
            for tag in tags["items"]:
                if tag.get("title") == tag_name:
                    tag_id = tag["id"]
                    break

            offset += 1

        result = nomad_sdk.add_tag_or_collection("tag", id, "asset", tag_name, tag_id, not tag_id)
        if not result:
            click.echo(json.dumps({ "error": "Error adding tag." }))
            sys.exit(1)
        7
        click.echo(json.dumps(result, indent=4))

    except Exception as e:
        click.echo(json.dumps({"error": f"Error adding tag: {e}"}))
        sys.exit(1)


import click
import json
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--playlist-id", required=True, help="The ID of the schedule the playlist video is to be updated from.")
@click.option("--item-id", required=True, help="The ID of the item to be updated.")
@click.option("--asset", required=False, help="The asset of the playlist video in JSON dict format.")
@click.pass_context
def update_playlist_video(ctx, playlist_id, item_id, asset):
    """Update playlist video"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        result = nomad_sdk.update_playlist_video(
            playlist_id,
            item_id,
            json.loads(asset) if asset else None
        )
        click.echo("Playlist video updated successfully.")
        click.echo(json.dumps(result, indent=4))

    except Exception as e:
        click.echo(json.dumps({"error": f"Error updating playlist video: {e}"}))
        sys.exit(1)


import click
import json
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--playlist-id", required=True, help="The ID of the playlist.")
@click.option("--video-asset", required=True, help="The video asset of the playlist video in JSON dict format.")
@click.option("--previous-item", required=False, help="The previous item of the playlist video.")
@click.pass_context
def create_playlist_video(ctx, playlist_id, video_asset, previous_item):
    """Create playlist video"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        result = nomad_sdk.create_playlist_video(
            playlist_id,
            json.loads(video_asset),
            previous_item
        )
        click.echo(json.dumps(result, indent=4))

    except Exception as e:
        click.echo(json.dumps({"error": f"Error creating playlist video: {e}"}))
        sys.exit(1)


import click
import json
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--id", required=True, help="The ID of the asset to import the annotations for.")
@click.option("--annotations", required=True, help="The annotations to import in JSON list format.")
@click.pass_context
def import_annotations(ctx, id, annotations):
    """Import annotations"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        nomad_sdk.import_annotations(id, json.loads(annotations))
        click.echo("Annotations imported successfully.")

    except Exception as e:
        click.echo(json.dumps({"error": f"Error importing annotations: {e}"}))
        sys.exit(1)


import click
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--schedule-id", required=True, help="The ID of the schedule to be started.")
@click.option("--skip-cleanup-on-failure", required=False, type=click.BOOL, help="Whether or not to skip cleanup on failure.")
@click.pass_context
def start_schedule(ctx, schedule_id, skip_cleanup_on_failure):
    """Start schedule"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        result = nomad_sdk.start_schedule(schedule_id, skip_cleanup_on_failure)
        click.echo("Schedule started successfully.")
        click.echo(result)

    except Exception as e:
        click.echo(json.dumps({"error": f"Error starting schedule: {e}"}))
        sys.exit(1)


import json

def check_token(config_path, nomad_sdk):
    with open(config_path, "r") as file:
        config = json.load(file)
    
    if not "token" in config:
        return

    if config["token"] == nomad_sdk.token:
        return
    
    config["token"] = nomad_sdk.token
    
    with open(config_path, "w") as file:
        json.dump(config, file, indent=4)


import click
import json
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--include-completed-uploads", required=True, type=click.BOOL, help="Whether to include completed uploads or not.")
@click.pass_context
def get_user_uploads(ctx, include_completed_uploads):
    """Get user uploads"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        result = nomad_sdk.get_user_uploads(include_completed_uploads)
        click.echo(json.dumps(result, indent=4))

    except Exception as e:
        click.echo(json.dumps({"error": f"Error getting user uploads: {e}"}))
        sys.exit(1)


import click
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--schedule-id", required=True, help="The ID of the schedule to be published.")
@click.option("--number-of-locked-days", required=True, type=click.INT, help="The number of locked days of the intelligent schedule.")
@click.pass_context
def publish_intelligent_schedule(ctx, schedule_id, number_of_locked_days):
    """Publish intelligent schedule"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        result = nomad_sdk.publish_intelligent_schedule(schedule_id, number_of_locked_days)
        click.echo("Intelligent schedule published successfully.")
        click.echo(result)

    except Exception as e:
        click.echo(json.dumps({"error": f"Error publishing intelligent schedule: {e}"}))
        sys.exit(1)


import click
import json
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--id", required=True, help="The ID of the asset to update the annotation for.")
@click.option("--annotation-id", required=True, help="The ID of the annotation.")
@click.option("--start-time-code", required=True, help="The start time code of the annotation. Format: hh:mm:ss;ff.")
@click.option("--end-time-code", required=True, help="The end time code of the annotation. Format: hh:mm:ss;ff.")
@click.option("--title", required=False, help="The title of the annotation.")
@click.option("--summary", required=False, help="The summary of the annotation.")
@click.option("--description", required=False, help="The description of the annotation.")
@click.pass_context
def update_annotation(ctx, id, annotation_id, start_time_code, end_time_code, title, summary, description):
    """Update annotation"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        result = nomad_sdk.update_annotation(
            id,
            annotation_id,
            start_time_code,
            end_time_code,
            title,
            summary,
            description
        )
        click.echo("Annotation updated successfully.")
        click.echo(json.dumps(result, indent=4))

    except Exception as e:
        click.echo(json.dumps({"error": f"Error updating annotation: {e}"}))
        sys.exit(1)


import click
import json
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--schedule-id", required=True, help="The ID of the schedule the asset item is to be added to.")
@click.option("--asset", required=True, help="The asset of the schedule item asset in JSON dict format.")
@click.option("--days", required=True, help="The days of the schedule item asset in JSON list format.")
@click.option("--duration-time-code", required=True, help="The duration time between time_code and end_time_code. Format: hh:mm:ss;ff.")
@click.option("--end-time-code", required=True, help="The end time code of the schedule item asset. Format: hh:mm:ss;ff.")
@click.option("--previous-item", required=False, help="The previous item of the schedule item asset.")
@click.option("--time-code", required=True, help="The time code of the schedule item asset. Format: hh:mm:ss;ff.")
@click.pass_context
def create_schedule_item_asset(ctx, schedule_id, asset, days, duration_time_code, end_time_code, previous_item, time_code):
    """Create schedule item asset"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        result = nomad_sdk.create_schedule_item_asset(
            schedule_id,
            json.loads(asset),
            json.loads(days),
            duration_time_code,
            end_time_code,
            previous_item,
            time_code
        )
        click.echo(json.dumps(result, indent=4))

    except Exception as e:
        click.echo(json.dumps({"error": f"Error creating schedule item asset: {e}"}))
        sys.exit(1)


import click
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--live-operator-id", required=True, help="The ID of the live operator.")
@click.pass_context
def start_segment(ctx, live_operator_id):
    """Start segment"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        nomad_sdk.start_segment(live_operator_id)
        click.echo("Segment started successfully.")

    except Exception as e:
        click.echo(json.dumps({"error": f"Error starting segment: {e}"}))
        sys.exit(1)


import click
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--live-channel-id", required=True, help="The ID of the live channel.")
@click.pass_context
def start_output_tracking(ctx, live_channel_id):
    """Start output tracking for live channel"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        nomad_sdk.start_output_tracking(live_channel_id)
        click.echo("Output tracking started successfully.")

    except Exception as e:
        click.echo(json.dumps({"error": f"Error starting output tracking: {e}"}))
        sys.exit(1)


import click
import json
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--id", required=True, help="The ID of the asset to update.")
@click.option("--display-name", required=False, help="The display name of the asset.")
@click.option("--display-date", required=False, help="The display date of the asset.")
@click.option("--available-start-date", required=False, help="The available start date of the asset.")
@click.option("--available-end-date", required=False, help="The available end date of the asset.")
@click.option("--custom-properties", required=False, help="The custom properties of the asset in JSON dict format.")
@click.pass_context
def update_asset(ctx, id, display_name, display_date, available_start_date, available_end_date, custom_properties):
    """Update asset"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        result = nomad_sdk.update_asset(
            id,
            display_name,
            display_date,
            available_start_date,
            available_end_date,
            json.loads(custom_properties) if custom_properties else None
        )
        click.echo("Asset updated successfully.")
        click.echo(json.dumps(result, indent=4))

    except Exception as e:
        click.echo(json.dumps({"error": f"Error updating asset: {e}"}))
        sys.exit(1)


import click
import json
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--schedule-id", required=True, help="The ID of the schedule the search filter item is to be added to.")
@click.option("--collections", required=False, help="The collections of the schedule item search filter in JSON list format.")
@click.option("--days", required=True, help="The days of the schedule item search filter in JSON list format.")
@click.option("--duration-time-code", required=True, help="The duration time between time_code and end_time_code. Format: hh:mm:ss;ff.")
@click.option("--end-search-date", required=False, help="The end search date of the schedule item search filter. Format: yyyy-MM-dd.THH:MM:SS.FFFZ.")
@click.option("--end-search-duration-in-minutes", required=True, type=click.INT, help="The end search duration in minutes of the schedule item search filter.")
@click.option("--end-time-code", required=True, help="The end time code of the schedule item search filter. Format: hh:mm:ss;ff.")
@click.option("--previous-item", required=False, help="The previous item of the schedule item search filter.")
@click.option("--related-contents", required=False, help="The related contents of the schedule item search filter in JSON list format.")
@click.option("--search-date", required=False, help="The search date of the schedule item search filter. Format: yyyy-MM-dd.THH:MM:SS.FFFZ.")
@click.option("--search-duration-in-minutes", required=True, type=click.INT, help="The search duration in minutes of the schedule item search filter.")
@click.option("--search-filter-type", required=True, type=click.INT, help="The search filter type of the schedule item search filter. Values: Random: 1, Random within a Date Range: 2, Newest: 3, Newest Not Played: 4")
@click.option("--tags", required=True, help="The tags of the schedule item search filter in JSON list format.")
@click.option("--time-code", required=True, help="The time code of the schedule item search filter. Format: hh:mm:ss;ff.")
@click.pass_context
def create_schedule_item_search_filter(ctx, schedule_id, collections, days, duration_time_code, end_search_date, end_search_duration_in_minutes, end_time_code, previous_item, related_contents, search_date, search_duration_in_minutes, search_filter_type, tags, time_code):
    """Create schedule item search filter"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        result = nomad_sdk.create_schedule_item_search_filter(
            schedule_id,
            json.loads(collections) if collections else None,
            json.loads(days),
            duration_time_code,
            end_search_date,
            end_search_duration_in_minutes,
            end_time_code,
            previous_item,
            json.loads(related_contents) if related_contents else None,
            search_date,
            search_duration_in_minutes,
            search_filter_type,
            json.loads(tags),
            time_code
        )
        click.echo(json.dumps(result, indent=4))

    except Exception as e:
        click.echo(json.dumps({"error": f"Error creating schedule item search filter: {e}"}))
        sys.exit(1)


from nomad_media_cli.helpers.utils import initialize_sdk

import click
import json
import sys

@click.command()
@click.pass_context
def list_buckets(ctx):
    """List buckets"""    
    
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:    
        filter = [{
            "fieldName": "assetType",
            "operation": "equals",
            "values": 5
        }]

        results = nomad_sdk.search(None, None, None, filter, None, None, None, None, None, None, None, None, None, None, None)

        result_names = [result["identifiers"]["name"] for result in results["items"]]
        
        click.echo(json.dumps(result_names, indent=4))

    except Exception as e:
        click.echo(json.dumps({ "error": f"Error listing buckets: {e}" }))
        sys.exit(1)


from nomad_media_cli.helpers.utils import initialize_sdk
import click
import json
import sys

@click.command()
@click.option("--config-type", required=True, type=click.INT, help="The type of config to get. 1 - Admin, 2 - Lambda, 3 - Groundtruth")
@click.pass_context
def get_config(ctx, config_type):
    """Get the configuration"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        if config_type < 1 or config_type > 3:
            click.echo(json.dumps({ "error": "Invalid config type. Must be 1, 2, or 3." }))
            sys.exit(1)        

        result = nomad_sdk.get_config(config_type)
        click.echo(json.dumps(result, indent=4))

    except Exception as e:
        click.echo(json.dumps({ "error": f"Error getting configuration: {e}" }))
        sys.exit(1)


import click
import json
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--id", required=True, help="The ID of the asset to move.")
@click.option("--destination-folder-id", required=True, help="The destination folder ID of the move.")
@click.option("--name", required=False, help="The name of the asset when moved.")
@click.option("--batch-action", required=False, help="The batch action of the move in JSON dict format.")
@click.option("--content-definition-id", required=False, help="The content definition ID of the move.")
@click.option("--schema-name", required=False, help="The schema name of the move.")
@click.option("--resolver-exempt", required=False, type=click.BOOL, help="The resolver exempt of the move.")
@click.pass_context
def move_asset(ctx, id, destination_folder_id, name, batch_action, content_definition_id, schema_name, resolver_exempt):
    """Move asset"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        result = nomad_sdk.move_asset(
            id,
            destination_folder_id,
            name,
            json.loads(batch_action) if batch_action else None,
            content_definition_id,
            schema_name,
            resolver_exempt
        )
        click.echo("Asset moved successfully.")
        click.echo(json.dumps(result, indent=4))

    except Exception as e:
        click.echo(json.dumps({"error": f"Error moving asset: {e}"}))
        sys.exit(1)


import click
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--content-id", required=True, help="The ID of the event to delete.")
@click.option("--content-definition-id", required=True, help="The content definition ID of the event to delete.")
@click.pass_context
def delete_event(ctx, content_id, content_definition_id):
    """Delete event"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        nomad_sdk.delete_event(content_id, content_definition_id)
        click.echo("Event deleted successfully.")

    except Exception as e:
        click.echo(json.dumps({"error": f"Error deleting event: {e}"}))
        sys.exit(1)


import click
import json
import sys

@click.command()
@click.option("--bucket", required=True, help="Bucket name")
@click.pass_context
def set_default_bucket(ctx , bucket):
    """Set the default bucket"""

    config_path = ctx.obj.get("config_path")

    try:
        with open(config_path, "r") as file:
            config = json.load(file)
        
        config["bucket"] = bucket

        with open(config_path, "w") as file:
            json.dump(config, file, indent=4)
    
    except Exception as e:
        click.echo(json.dumps({ "error": f"Error setting bucket: {e}" }))       
        sys.exit(1)


import click
import json
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--id", required=True, help="The ID of the asset of the annotation.")
@click.option("--url", help="The Nomad URL of the Asset (file or folder) to delete the annotation for (bucket::object-key).")
@click.option("--object-key", help="Object-key of the Asset (file or folder) to delete the annotation for. This option assumes the default bucket that was previously set with the `set-bucket` command.")
@click.option("--annotation-id", required=True, help="The ID of the annotation.")
@click.pass_context
def delete_annotation(ctx, id, url, object_key, annotation_id):
    """Delete annotation"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    if url or object_key:
        if url and "::" not in url:
            click.echo(json.dumps({ "error": "Please provide a valid path or set the default bucket." }))               
            sys.exit(1)
        if object_key:
            if "bucket" in ctx.obj:
                url = f"{ctx.obj['bucket']}::{object_key}"
            else:
                click.echo(json.dumps({ "error": "Please set bucket using `set-bucket` or use url." }))
                sys.exit(1)
                
        url_search_results = nomad_sdk.search(None, None, None, [{
            "fieldName": "url",
            "operator": "equals",
            "values": url
        }], None, None, None, None, None, None, None, None, None)
        
        if not url_search_results or len(url_search_results["items"] == 0):
            click.echo(json.dumps({ "error": f"URL {url} not found." }))
            sys.exit(1)
            
        id = url_search_results["items"][0]["id"]
    try:
        result = nomad_sdk.delete_annotation(id, annotation_id)
        click.echo("Annotation deleted successfully.")
        click.echo(json.dumps(result, indent=4))

    except Exception as e:
        click.echo(json.dumps({"error": f"Error deleting annotation: {e}"}))
        sys.exit(1)


from nomad_media_cli.commands.cli_commands.login import login

import click
import json
import os

@click.command()
@click.option("--service-api-url", required=True, help="API URL for the service")
@click.option("--api-type", default="admin", type=click.Choice(['admin', 'portal']), help="API type [admin|portal]. Default is admin")
@click.option("--debug", default=False, type=click.BOOL, help="Enable debug mode. Default is false")
@click.option("--username", help="Username credential")
@click.option("--password", help="Password credential")
@click.pass_context
def init(ctx, service_api_url, api_type, debug, username, password):
    """Initialize the SDK and save configuration"""
    
    config_path = ctx.obj["config_path"]
    config_dir = os.path.dirname(config_path)
    os.makedirs(config_dir, exist_ok=True)
    
    if not service_api_url.startswith("https://"):
        service_api_url = f"https://{service_api_url}"
        
    debug = debug or False
    
    config = {
        "serviceApiUrl": service_api_url,
        "apiType": api_type,
        "debugMode": debug,
        "disableLogging": not debug
    }
    
    if (username or password) and not (username and password):
        click.echo("Please provide both username and password if you are providing credentials.")
        return
    
    try:
        with open(config_path, "w") as file:
            json.dump(config, file, indent=4)
            
        if not username and not password:
            click.echo(json.dumps({ "message": "Successfully initialized the CLI" }))
            
        if username and password:
            ctx.invoke(login, username=username, password=password)
            
    except Exception as e:
        click.echo(json.dumps({ "error": f"Error saving configuration: {e}" }))


import click
import json
import sys
from nomad_media_cli.helpers.utils import initialize_sdk
from nomad_media_cli.helpers.get_id_from_url import get_id_from_url

@click.command()
@click.option("--id", help="The ID of the asset to get the details for.")
@click.option("--url", help="The Nomad URL of the Asset (file or folder) to get the details for (bucket::object-key).")
@click.option("--object-key", help="Object-key of the Asset (file or folder) to get the details for. This option assumes the default bucket that was previously set with the `set-bucket` command.")
@click.pass_context
def get_asset_details(ctx, id, url, object_key):
    """Get asset details"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]
    
    if not id and not url and not object_key:
        click.echo(json.dumps({"error": "Please provide an id, url, or object-key."}))
        sys.exit(1)
    
    if url or object_key:
        id = get_id_from_url(ctx, url, object_key, nomad_sdk)

    try:
        result = nomad_sdk.get_asset_details(id)
        if not result:
            click.echo(json.dumps({"error": f"Asset with id {id} not found."}))
            sys.exit(1)        

        click.echo(json.dumps(result, indent=4))

    except Exception as e:
        click.echo(json.dumps({"error": f"Error getting asset details: {e}"}))
        sys.exit(1)


from nomad_media_cli.helpers.utils import get_config
from nomad_media_cli.helpers.utils import initialize_sdk

import click
import json
import sys

@click.command()
@click.pass_context
def logout(ctx):
    """Logs out of the service"""

    initialize_sdk(ctx)

    nomad_sdk = ctx.obj["nomad_sdk"]
    get_config(ctx)

    config = ctx.obj["config"]
    config_path = ctx.obj["config_path"]

    try:
        del config["token"]
        del config["refresh_token_val"]
        del config["expiration_seconds"]
        del config["id"]
        
        with open(config_path, "w") as file:
            json.dump(config, file, indent=4)
            
        nomad_sdk.logout()
        
        click.echo(json.dumps({ "message": "Successfully logged out of the CLI." }))
            
    except Exception as e:
        click.echo(json.dumps({ "error": f"Error logging out: {e}" }))
        sys.exit(1)


import click
import json
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--id", required=True, help="The ID of the asset to transcribe.")
@click.option("--transcript-id", required=True, help="The ID of the transcript.")
@click.option("--transcript", required=False, help="The transcript of the transcribe in JSON list format.")
@click.pass_context
def transcribe_asset(ctx, id, transcript_id, transcript):
    """Transcribe asset"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        result = nomad_sdk.transcribe_asset(
            id,
            transcript_id,
            json.loads(transcript) if transcript else None
        )
        click.echo("Asset transcribed successfully.")
        click.echo(json.dumps(result, indent=4))

    except Exception as e:
        click.echo(json.dumps({"error": f"Error transcribing asset: {e}"}))
        sys.exit(1)


import click
import json
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--id", required=True, help="The ID of the asset to be clipped.")
@click.option("--start-time-code", required=True, help="The start time code of the asset. Format: hh:mm:ss;ff.")
@click.option("--end-time-code", required=True, help="The end time code of the asset. Format: hh:mm:ss;ff.")
@click.option("--title", required=True, help="The title of the asset.")
@click.option("--output-folder-id", required=True, help="The output folder ID of the asset.")
@click.option("--tags", required=False, help="The tags of the asset in JSON list format.")
@click.option("--collections", required=False, help="The collections of the asset in JSON list format.")
@click.option("--related-contents", required=False, help="The related contents of the asset in JSON list format.")
@click.option("--video-bitrate", required=False, type=click.INT, help="The video bitrate of the asset.")
@click.option("--audio-tracks", required=False, help="The audio tracks of the asset in JSON list format.")
@click.pass_context
def clip_asset(ctx, id, start_time_code, end_time_code, title, output_folder_id, tags, collections, related_contents, video_bitrate, audio_tracks):
    """Clip asset"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        result = nomad_sdk.clip_asset(
            id,
            start_time_code,
            end_time_code,
            title,
            output_folder_id,
            json.loads(tags) if tags else None,
            json.loads(collections) if collections else None,
            json.loads(related_contents) if related_contents else None,
            video_bitrate,
            json.loads(audio_tracks) if audio_tracks else None
        )
        click.echo("Asset clipped successfully.")
        click.echo(json.dumps(result, indent=4))

    except Exception as e:
        click.echo(json.dumps({"error": f"Error clipping asset: {e}"}))
        sys.exit(1)


import click
import sys, json

def get_content_definition_id(ctx, content_definition_name):
    """Get the content definition ID from the content definition name"""
    nomad_sdk = ctx.obj["nomad_sdk"]
    content_definitions = nomad_sdk.get_content_definitions(None, None, None, None, None)
    
    content_definition = next(filter(lambda x: x["properties"]["title"] == content_definition_name, content_definitions["items"]), None)
    
    if not content_definition:
        click.echo(json.dumps({ "error": "Content definition not found." }))
        sys.exit(1)
        
    return content_definition["contentDefinitionId"]



import click
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--id", required=True, help="The ID of the asset to record the asset tracking beacon for.")
@click.option("--tracking-event", required=True, help="The tracking event of the asset tracking beacon.")
@click.option("--live-channel-id", required=True, help="The live channel ID of the asset tracking beacon.")
@click.option("--content-id", required=False, help="Optional content ID to track along with required asset ID.")
@click.option("--second", required=True, type=click.INT, help="Second mark into the video/ad.")
@click.pass_context
def records_asset_tracking_beacon(ctx, id, tracking_event, live_channel_id, content_id, second):
    """Record asset tracking beacon"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        nomad_sdk.records_asset_tracking_beacon(id, tracking_event, live_channel_id, content_id, second)
        click.echo("Asset tracking beacon recorded successfully.")

    except Exception as e:
        click.echo(json.dumps({"error": f"Error recording asset tracking beacon: {e}"}))
        sys.exit(1)


import click
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--event-id", required=True, help="The ID of the event to stop the live schedule of.")
@click.pass_context
def stop_live_schedule(ctx, event_id):
    """Stop live schedule of event"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        nomad_sdk.stop_live_schedule(event_id)
        click.echo("Live schedule stopped successfully.")

    except Exception as e:
        click.echo(json.dumps({"error": f"Error stopping live schedule: {e}"}))
        sys.exit(1)


from nomad_media_cli.helpers.utils import initialize_sdk
import click
import json
import sys

@click.command()
@click.pass_context
def create_content_definition(ctx):
    """Create a new content definition"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        result = nomad_sdk.create_content_definition()
        click.echo(json.dumps(result, indent=4))

    except Exception as e:
        click.echo(json.dumps({ "error": f"Error creating content definition: {e}" }))
        sys.exit(1)


import click
import json
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--live-output-id", required=True, help="The ID of the live output profile.")
@click.option("--name", required=False, help="The name of the live output profile.")
@click.option("--output-type", required=False, help="The type of the live output profile in JSON dict format.")
@click.option("--enabled", required=False, type=click.BOOL, help="Indicates if the live output profile is enabled.")
@click.option("--audio-bitrate", required=False, type=click.INT, help="The audio bitrate of the live output profile.")
@click.option("--output-stream-key", required=False, help="The output stream key of the live output profile.")
@click.option("--output-url", required=False, help="The output URL of the live output profile.")
@click.option("--secondary-output-stream-key", required=False, help="The secondary output stream key of the live output profile.")
@click.option("--secondary-url", required=False, help="The secondary URL of the live output profile.")
@click.option("--video-bitrate", required=False, type=click.INT, help="The video bitrate of the live output profile.")
@click.option("--video-bitrate-mode", required=False, help="The video bitrate mode of the live output profile. The modes are CBR and VBR.")
@click.option("--video-codec", required=False, help="The video codec of the live output profile. The codecs are H264 and H265.")
@click.option("--video-frames-per-second", required=False, type=click.INT, help="The video frames per second of the live output profile.")
@click.option("--video-height", required=False, type=click.INT, help="The video height of the live output profile.")
@click.option("--video-width", required=False, type=click.INT, help="The video width of the live output profile.")
@click.pass_context
def update_live_output_profile(ctx, live_output_id, name, output_type, enabled, audio_bitrate, output_stream_key, output_url, secondary_output_stream_key, secondary_url, video_bitrate, video_bitrate_mode, video_codec, video_frames_per_second, video_height, video_width):
    """Update live output profile"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        result = nomad_sdk.update_live_output_profile(
            live_output_id,
            name,
            json.loads(output_type) if output_type else None,
            enabled,
            audio_bitrate,
            output_stream_key,
            output_url,
            secondary_output_stream_key,
            secondary_url,
            video_bitrate,
            video_bitrate_mode,
            video_codec,
            video_frames_per_second,
            video_height,
            video_width
        )
        click.echo(json.dumps(result, indent=4))

    except Exception as e:
        click.echo(json.dumps({"error": f"Error updating live output profile: {e}"}))
        sys.exit(1)


from nomad_media_cli.helpers.utils import initialize_sdk
import click
import sys

@click.command()
@click.option("--session-id", required=True, help="The ID of the session.")
@click.option("--content-id", required=True, help="The ID of the content.")
@click.option("--content-definition-id", required=True, help="The ID of the content definition.")
@click.option("--deactivate", type=click.BOOL, required=True, help="The deactivate flag.")
@click.pass_context
def deactivate_content_user_track(ctx, session_id, content_id, content_definition_id, deactivate):
    """Deactivate a content user track"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        nomad_sdk.deactivate_content_user_track(session_id, content_id, content_definition_id, deactivate)
        click.echo("Content user track deactivated successfully")

    except Exception as e:
        click.echo(json.dumps({ "error": f"Error deactivating content user track: {e}" }))
        sys.exit(1)


import click
import json
import sys

@click.command()
@click.option("--service-api-url", help="API URL for the service")
@click.option("--api-type", type=click.Choice(['admin', 'portal']), help="API type [admin|portal].")
@click.option("--debug", type=click.BOOL, help="Enable debug mode.")
@click.pass_context
def update_config(ctx, service_api_url, api_type, debug):
    """Update the configuration"""
    
    config_path = ctx.obj["config_path"]
    
    if service_api_url and not service_api_url.startswith("https://"):
        service_api_url = f"https://{service_api_url}"

    try:
        with open(config_path, "r") as file:
            config = json.load(file)
        
        if service_api_url:
            config["serviceApiUrl"] = service_api_url
            
        if api_type:
            config["apiType"] = api_type
            
        if debug is not None:
            debug = debug or False    

            config["debugMode"] = debug
            config["disableLogging"] = not debug

        with open(config_path, "w") as file:
            json.dump(config, file, indent=4)

        click.echo(json.dumps({ "message": "Successfully updated the CLI configuration" }))
    
    except Exception as e:
        click.echo(json.dumps({ "error": f"Error updating configuration: {e}" }))
        sys.exit(1)


import click
import json
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--id", required=True, help="The ID of the asset to restore.")
@click.pass_context
def restore_asset(ctx, id):
    """Restore asset"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        result = nomad_sdk.restore_asset(id)
        click.echo("Asset restored successfully.")
        click.echo(json.dumps(result, indent=4))

    except Exception as e:
        click.echo(json.dumps({"error": f"Error restoring asset: {e}"}))
        sys.exit(1)


import click
import json
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--action-arguments", required=True, help="The action arguments of the start in JSON dict format.")
@click.option("--target-ids", required=True, help="The target IDs of the start in JSON list format.")
@click.pass_context
def start_workflow(ctx, action_arguments, target_ids):
    """Start workflow"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        result = nomad_sdk.start_workflow(json.loads(action_arguments), json.loads(target_ids))
        click.echo("Workflow started successfully.")
        click.echo(json.dumps(result, indent=4))

    except Exception as e:
        click.echo(json.dumps({"error": f"Error starting workflow: {e}"}))
        sys.exit(1)


import click
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--live-output-id", required=True, help="The ID of the live output profile.")
@click.pass_context
def delete_live_output_profile(ctx, live_output_id):
    """Delete live output profile"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        nomad_sdk.delete_live_output_profile(live_output_id)
        click.echo("Live output profile deleted successfully.")

    except Exception as e:
        click.echo(json.dumps({"error": f"Error deleting live output profile: {e}"}))
        sys.exit(1)


import os, json
import click
import sys
import uuid
import concurrent.futures
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--source", help="Local OS file or folder path specifying the files or folders to upload. For example: file.jpg or folderName/file.jpg or just folderName.")
@click.option("--id", help="Nomad ID of the Asset Folder to upload the source file(s) and folder(s) into.")
@click.option("--url", help="The Nomad URL of the Asset (file or folder) to list the assets for (bucket::object-key).")
@click.option("--object-key", help="Object-key of the Asset (file or folder) to list the assets for. This option assumes the default bucket that was previously set with the `set-bucket` command.")
@click.option("-r", "--recursive", is_flag=True, help="Recursively upload a folder")
@click.pass_context
def upload_assets(ctx, source, id, url, object_key, recursive):
    """Upload assets"""
    
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]
    config = ctx.obj["config"]

    if not id and not url and not object_key:
        click.echo(json.dumps({"error": "Please provide an id, url, or object-key."}))
        sys.exit(1)

    try:
        parent_id = None        

        if not source:
            click.echo(json.dumps({ "error": "Please provide a file or folder to upload." }))
            sys.exit(1)
            
        if not os.path.exists(source):
            click.echo(json.dumps({ "error": "Source path does not exist." }))
            sys.exit(1)
            
        if not id and not url and not object_key:
            click.echo(json.dumps({ "error": "Please provide a parent id, url, or objectKey." }))
            sys.exit(1)
            
        if id and not is_valid_uuid(id):
            click.echo(json.dumps({ "error": "Please provide a valid UUID." }))
            sys.exit(1)

        if url and "::" not in url:
            click.echo(json.dumps({ "error": "Please provide a valid path." }))
            sys.exit(1)

        if object_key:
            if not object_key.endswith("/"):            
                object_key = f"{object_key}/"
                
            if "bucket" in config:
                url = f"{config['bucket']}::{object_key}"
            else:
                click.echo(json.dumps({ "error": "Please set bucket using `set-bucket` or use url." }))
                sys.exit(1)
            
        if url:
            if not url.endswith("/"):
                url = f"{url}/"            

            id_search_response = nomad_sdk.search(None, None, None,
                [
                    {
                        "fieldName": "url",
                        "operator": "equals",
                        "values": url
                    }
                ], None, None, None, None, None, None, None, None, None, None, None)
            
            if len(id_search_response["items"]) == 0:
                click.echo(json.dumps({ "error": "No asset found with the provided URL." }))
                sys.exit(1)
                
            parent_id = id_search_response["items"][0]["identifiers"]["parentId"]
        elif id:
            parent_id = id
            
        response = nomad_sdk.get_asset(parent_id)
        if not response:
            click.echo(json.dumps({ "error": f"Parent folder not found: {parent_id}." }))
            sys.exit(1)
            
        if response["assetType"] != 1:
            click.echo(json.dumps({ "error": "Asset must be a folder" }))
            sys.exit(1)
            
        if os.path.isdir(source):
            if recursive:

                source_name = os.path.basename(source)
                folder_id = find_folder_id(parent_id, source_name, 1, nomad_sdk)                  

                folder_id_map = {source: folder_id}
                
                for root, dirs, files in os.walk(source):
                    folder_name = os.path.basename(root)
                    folder_id = folder_id_map.get(root)

                    if not folder_id:
                        parent_folder_id = folder_id_map[os.path.dirname(root)]

                        folder_id = find_folder_id(parent_folder_id, folder_name, 1, nomad_sdk)

                        folder_id_map[root] = folder_id

                    for name in files:
                        if find_folder_id(folder_id, name, 2, nomad_sdk):
                            continue

                        file_path = os.path.join(root, name)
                        if os.path.getsize(file_path) == 0:
                            continue
                        
                        try:
                            upload_with_retry(file_path, folder_id, nomad_sdk)
                        except Exception as e:
                            click.echo(json.dumps({ "error": f"Error uploading file: {file_path} - {e}" }))
                    
            else:
                click.echo(json.dumps({ "error": "Please use the --recursive option to upload directories." }))
                sys.exit(1)
        else:
            if os.path.getsize(source) == 0:
                click.echo(json.dumps({ "error": "File is empty." }))
                sys.exit(1)                
            
            asset_id = upload_with_retry(source, parent_id, nomad_sdk)
            click.echo(json.dumps(asset_id, indent=4))
                
    except Exception as e:
        click.echo(json.dumps({ "error": f"Error uploading assets: {e}" }))
        sys.exit(1)
        
def find_folder_id(parent_id, folder_name, asset_type, nomad_sdk):
    offset = 0
    folder_id = None
    while True:
        nomad_folders = nomad_sdk.search(None, offset, None,
            [
                {
                    "fieldName": "parentId",
                    "operator": "equals",
                    "values": parent_id
                },
                {
                    "fieldName": "assetType",
                    "operator": "equals",
                    "values": asset_type
                }
            ],
            None, None, None, None, None, None, None, None, None, None, None)                     

        if len(nomad_folders["items"]) == 0:
            break

        folder = next((nomad_folder for nomad_folder in nomad_folders["items"] if nomad_folder["title"] == folder_name), None)
        if folder:
            folder_id = folder["id"]
            break

        offset += 1
        
    if not folder_id and asset_type == 1:
        folder = nomad_sdk.create_folder_asset(parent_id, folder_name)
        folder_id = folder["id"]
        
    return folder_id
        
def upload_with_retry(file_path, folder_id, nomad_sdk, retries=3):
    for attempt in range(retries):
        try:
            response = nomad_sdk.upload_asset(None, None, None, "replace", file_path, folder_id, None)
            return response
        except Exception as e:
            if attempt == retries - 1:
                raise e
            
def is_valid_uuid(val):
    try:
        uuid.UUID(str(val))
        return True
    except ValueError:
        return False


import click
import json
import sys
from nomad_media_cli.helpers.utils import initialize_sdk
from nomad_media_cli.helpers.get_id_from_url import get_id_from_url

@click.command()
@click.option("--id", help="The id of the asset.")
@click.option("--url", help="The Nomad URL of the Asset (file or folder) to add the tag to (bucket::object-key).")
@click.option("--object-key", help="Object-key of the Asset (file or folder) to add the tag to. This option assumes the default bucket that was previously set with the `set-bucket` command.")
@click.option("--name", help="The display name of the asset.")
@click.option("--date", help="The display date of the asset.")
@click.option("--properties", help="The custom properties of the asset. Must be in JSON format. Example: '{\"key\": \"value\"}'")
@click.pass_context
def add_asset_properties(ctx, id, url, object_key, name, date, properties):
    """Add custom properties to asset"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]
        
    if not id and not url and not object_key:
        click.echo(json.dumps({"error": "Please provide either an id, url or object-key"}))
        sys.exit(1)

    if url or object_key:
        id = get_id_from_url(ctx, url, object_key, nomad_sdk)

    try:
        try:
            if properties:
                properties = json.loads(properties)
        except:
            click.echo(json.dumps({ "error": "Custom properties must be in JSON format." }))
            sys.exit(1)    

        result = nomad_sdk.add_custom_properties(id, name, date, properties)
        if not result:
            click.echo(json.dumps({ "error": "Error adding custom properties to asset." }))
            sys.exit(1)
        click.echo(json.dumps({ "message": "Custom properties added to asset." }))
    except Exception as e:
        click.echo(json.dumps({ "error": f"Error adding custom properties to asset." }))


import json
import os
import unittest
from click.testing import CliRunner
from nomad_media_cli.cli import cli
from nomad_media_cli.helpers.get_content_definition_id import get_content_definition_id

class TestAssetBase(unittest.TestCase):
    
    @classmethod
    def setUpClass(cls):
        cls.runner = CliRunner()
#
        with open("nomad_media_cli/tests/test-config.json", "r") as file:
            test_config = json.load(file)
            cls.test_dir_id = test_config["testDirId"]
        
        #result = cls.runner.invoke(cli, [
        #    "upload-assets", 
        #    "--source", "README.md",
        #    "--id", cls.test_dir_id
        #])
        #
        #if result.exit_code != 0:
        #    raise Exception(f"Failed to upload asset: {result.output}")
        #
        #cls.asset_id = result.output.replace('"', "").strip()
        cls.asset_id = "20133c0c-19ab-4200-8609-76d9115a9304"

    @classmethod
    def setup_config(cls):
        config_path_result = cls.runner.invoke(cli, ["list-config-path"])
        if config_path_result.exit_code != 0:
            raise Exception(f"Need to run `nomad-media-cli init` before running tests")
        
        config_path = json.loads(config_path_result.output.strip())

        with open(config_path["path"], "r") as file:
            config = json.load(file)
            cls.config = config
            cls.config_path = config_path["path"]
            
    #@classmethod
    #def tearDownClass(cls):
    #    result = cls.runner.invoke(cli, [
    #        "delete-asset", 
    #        "--id", cls.asset_id
    #    ])
    #    
    #    if result.exit_code != 0:
    #        raise Exception(f"Failed to delete asset: {result.output}")
    #    
    #    print(f"Deleted asset with id: {cls.asset_id}")
        
class TestAssetUpload(TestAssetBase):
    """Tests for uploading assets"""

    #@classmethod
    #def tearDownClass(cls):
    #    result = cls.runner.invoke(cli, [
    #        "delete-asset", 
    #        "--id", cls.asset_upload_id
    #    ])
    #    
    #    if result.exit_code != 0:
    #        raise Exception(f"Failed to delete asset: {result.output}")
    #    
    #    print(f"Deleted asset with id: {cls.asset_upload_id}")
    
    def test_upload_asset(self):
        """Test asset is uploaded successfully"""
        result = self.runner.invoke(cli, [
            "upload-assets", 
            "--source", "nomad_media_cli/tests/test_files/vid.mp4",
            "--id", self.test_dir_id
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        asset_id = result.output.replace('"', "").strip()
        self.assertIsNotNone(asset_id)
        
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", asset_id
        ])
        
        self.assertEqual(asset_details_result.exit_code, 0)
        
        self.asset_upload_id = asset_id
        
    def test_upload_asset_invalid_file(self):
        """Test invalid file returns an error"""
        result = self.runner.invoke(cli, [
            "upload-assets", 
            "--source", "nomad_media_cli/tests/test_files/invalid-file",
            "--id", self.test_dir_id
        ])
        
        self.assertNotEqual(result.exit_code, 0)
        
    def test_upload_asset_empty_file(self):
        """Test empty file returns an error"""
        result = self.runner.invoke(cli, [
            "upload-assets", 
            "--source", "nomad_media_cli/tests/__init__.py",
            "--id", self.test_dir_id
        ])
        
        self.assertNotEqual(result.exit_code, 0)
        
    def test_upload_asset_invalid_id(self):
        """Test invalid ID returns an error"""
        result = self.runner.invoke(cli, [
            "upload-assets", 
            "--source", "nomad_media_cli/tests/test_files/vid.mp4",
            "--id", "invalid-id"
        ])
        
        self.assertNotEqual(result.exit_code, 0)
        
    def test_upload_asset_directory_flat(self):
        """Test uploading a directory of files with no subdirectories"""
        result = self.runner.invoke(cli, [
            "upload-assets", 
            "--source", "nomad_media_cli/tests/test_files",
            "--id", self.test_dir_id,
            "-r"
        ])
        
        self.assertEqual(result.exit_code, 0)
        num_files_in_dir = sum([len(files) + len(dirs) for _, dirs, files in os.walk("nomad_media_cli/tests/test_files")])
        
        list_assets_parent_result = self.runner.invoke(cli, [
            "list-assets", 
            "--id", self.test_dir_id
        ])
        
        self.assertEqual(list_assets_parent_result.exit_code, 0)
        
        output_json = json.loads(list_assets_parent_result.output)
        dir_id = next((item["id"] for item in output_json["items"] if item["name"] == "test_files/"), None)
        
        if not dir_id:
            self.fail("Directory not found")
            
        list_assets_result = self.runner.invoke(cli, [
            "list-assets", 
            "--id", dir_id
        ])

        self.assertEqual(list_assets_result.exit_code, 0)
        
        output_json = json.loads(list_assets_result.output)
        self.assertEqual(len(output_json["items"]), num_files_in_dir)
        
        self.asset_upload_id = dir_id
        
    def test_upload_asset_directory_small(self):
        """Test uploading a directory of files with a small number of files"""
        result = self.runner.invoke(cli, [
            "upload-assets", 
            "--source", "nomad_media_cli/tests",
            "--id", self.test_dir_id,
            "-r"
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        num_files_in_dir = sum([len(files) + len(dirs) for _, dirs, files in os.walk("nomad_media_cli/tests")])
        
        list_assets_parent_result = self.runner.invoke(cli, [
            "list-assets", 
            "--id", self.test_dir_id
        ])
        
        self.assertEqual(list_assets_parent_result.exit_code, 0)
        
        output_json = json.loads(list_assets_parent_result.output)
        dir_id = next((item["id"] for item in output_json["items"] if item["name"] == "tests/"), None)        

        list_assets_result = self.runner.invoke(cli, [
            "list-assets", 
            "--id", dir_id,
            "-r"
        ])
        
        self.assertEqual(list_assets_result.exit_code, 0)

        output_json = json.loads(list_assets_result.output)
        self.assertEqual(len(output_json["items"]), num_files_in_dir)
        
        self.asset_upload_id = dir_id
        
    def test_upload_asset_directory_large(self):
        """Test uploading a directory of files with a large number of files"""
        result = self.runner.invoke(cli, [
            "upload-assets", 
            "--source", "nomad_media_cli/commands",
            "--id", self.test_dir_id,
            "-r"
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        num_files_in_dir = sum([len(files) + len(dirs) for _, dirs, files in os.walk("nomad_media_cli/commands")])
        
        list_assets_parent_result = self.runner.invoke(cli, [
            "list-assets", 
            "--id", self.test_dir_id
        ])
        
        self.assertEqual(list_assets_parent_result.exit_code, 0)
        
        output_json = json.loads(list_assets_parent_result.output)
        dir_id = next((item["id"] for item in output_json["items"] if item["name"] == "commands/"), None)        

        list_assets_result = self.runner.invoke(cli, [
            "list-assets", 
            "--id", dir_id,
            "-r"
        ])
        
        self.assertEqual(list_assets_result.exit_code, 0)
        
        output_json = json.loads(list_assets_result.output)
        self.assertEqual(output_json["totalItemCount"], num_files_in_dir)
        
        self.asset_upload_id = dir_id
        
    def test_upload_asset_directory_not_recursive(self):
        """Test directory returns an error"""
        result = self.runner.invoke(cli, [
            "upload-assets", 
            "--source", "nomad_media_cli/tests/test_files",
            "--id", self.test_dir_id
        ])
        
        self.assertNotEqual(result.exit_code, 0)

class TestBucketCommands(TestAssetBase):
    # Test list buckets
    def test_list_buckets(self):
        
        result = self.runner.invoke(cli, ["list-buckets"])
        self.assertEqual(result.exit_code, 0)
        
        try:
            output_json = json.loads(result.output)
            self.assertTrue(isinstance(output_json, list))
        except json.JSONDecodeError:
            self.fail("Output is not valid JSON")
        
    # Test set default bucket
    def test_set_default_bucket(self):
        
        self.setup_config()
        bucket = self.config.get("bucket")
            
        buckets_response = self.runner.invoke(cli, ["list-buckets"])
        buckets = json.loads(buckets_response.output)

        if len(buckets) == 0:
            self.skipTest("No buckets available")

        result = self.runner.invoke(cli, [
            "set-default-bucket",
            "--bucket", buckets[0]])
        
        self.assertEqual(result.exit_code, 0)
        
        with open(self.config_path, "r") as file:
            config = json.load(file)
            new_config_bucket = config.get("bucket")
            
        self.assertEqual(new_config_bucket, buckets[0])
        
        if bucket:
            # Reset the bucket
            result = self.runner.invoke(cli, [
                "set-default-bucket",
                "--bucket", bucket])

class TestAssetList(TestAssetBase):
    """Tests for listing assets"""
    
    def test_list_assets_file_by_id(self):
        
        result = self.runner.invoke(cli, [
            "list-assets", 
            "--id", self.asset_id
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        try:
            output_json = json.loads(result.output)
            self.assertTrue(isinstance(output_json["items"], list))
            self.assertTrue(len(output_json["items"]) > 0)
        except json.JSONDecodeError:
            self.fail("Output is not valid JSON")
            
    def test_list_assets_folder_by_id(self):
        
        asset_parent_id = self.test_dir_id

        result = self.runner.invoke(cli, [
            "list-assets", 
            "--id", asset_parent_id
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        try:
            output_json = json.loads(result.output)
            self.assertTrue(isinstance(output_json["items"], list))
            self.assertTrue(len(output_json["items"]) > 0)
        except json.JSONDecodeError:
            self.fail("Output is not valid JSON")
            
    def test_list_assets_folder_recursive(self):
        
        asset_parent_id = self.test_dir_id

        result = self.runner.invoke(cli, [
            "list-assets", 
            "--id", asset_parent_id,
            "--recursive"
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        try:
            output_json = json.loads(result.output)
            self.assertTrue(isinstance(output_json["items"], list))
            self.assertTrue(len(output_json["items"]) > 0)
        except json.JSONDecodeError:
            self.fail("Output is not valid JSON")

    def test_list_assets_by_id_invalid(self):
        
        result = self.runner.invoke(cli, [
            "list-assets", 
            "--id", "invalid-id"
        ])
        
        self.assertNotEqual(result.exit_code, 0)

    def test_list_assets_file_by_url(self):
        
        list_buckets_result = self.runner.invoke(cli, ["list-buckets"])
        buckets = json.loads(list_buckets_result.output)
        
        if len(buckets) == 0:
            self.skipTest("No buckets available")
            
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.asset_id
        ])
        
        asset_details = json.loads(asset_details_result.output)
        url = asset_details["properties"]["url"]
            
        result = self.runner.invoke(cli, [
            "list-assets", 
            "--url", url
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        try:
            output_json = json.loads(result.output)
            self.assertTrue(isinstance(output_json["items"], list))
            self.assertTrue(len(output_json["items"]) > 0)
        except json.JSONDecodeError:
            self.fail("Output is not valid JSON")
            
    def test_list_assets_folder_by_url(self):
        
        list_buckets_result = self.runner.invoke(cli, ["list-buckets"])
        buckets = json.loads(list_buckets_result.output)
        
        if len(buckets) == 0:
            self.skipTest("No buckets available")
            
        asset_parent_id = self.test_dir_id
        asset_parent_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", asset_parent_id
        ])
        
        asset_parent_details = json.loads(asset_parent_details_result.output)
        url = asset_parent_details["properties"]["url"]
            
        result = self.runner.invoke(cli, [
            "list-assets", 
            "--url", url
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        try:
            output_json = json.loads(result.output)
            self.assertTrue(isinstance(output_json["items"], list))
            self.assertTrue(len(output_json["items"]) > 0)
        except json.JSONDecodeError:
            self.fail("Output is not valid JSON")

    def test_list_assets_by_url_invalid(self):
        
        result = self.runner.invoke(cli, [
            "list-assets", 
            "--url", "invalid-url"
        ])
        
        self.assertNotEqual(result.exit_code, 0)
            
    def test_list_assets_by_file_object_key(self):
        
        self.setup_config()

        bucket = self.config.get("bucket")
        if not bucket:
            self.skipTest("No default bucket set")
            
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.asset_id
        ])
        
        asset_details = json.loads(asset_details_result.output)
        object_key = asset_details["properties"]["originalObjectKey"]
            
        result = self.runner.invoke(cli, [
            "list-assets", 
            "--object-key", object_key
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        try:
            output_json = json.loads(result.output)
            self.assertTrue(isinstance(output_json["items"], list))
            self.assertTrue(len(output_json["items"]) > 0)
        except json.JSONDecodeError:
            self.fail("Output is not valid JSON")
            
    def test_list_assets_by_folder_object_key(self):
        
        self.setup_config()

        bucket = self.config.get("bucket")
        if not bucket:
            self.skipTest("No default bucket set")
            
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.asset_id
        ])
        
        asset_details = json.loads(asset_details_result.output)
        object_key = asset_details["properties"]["originalObjectKey"]
        
        result = self.runner.invoke(cli, [
            "list-assets", 
            "--object-key", object_key
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        try:
            output_json = json.loads(result.output)
            self.assertTrue(isinstance(output_json["items"], list))
            self.assertTrue(len(output_json["items"]) > 0)
        except json.JSONDecodeError:
            self.fail("Output is not valid JSON")
            
    def test_list_assets_by_object_key_no_bucket(self):
        
        self.setup_config()

        bucket = self.config.get("bucket")
        if bucket:
            self.skipTest("Default bucket set")
            
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.asset_id
        ])
        
        asset_details = json.loads(asset_details_result.output)
        object_key = asset_details["properties"]["originalObjectKey"]
        
        result = self.runner.invoke(cli, [
            "list-assets", 
            "--object-key", object_key
        ])
        
        self.assertNotEqual(result.exit_code, 0)
            
    def test_list_assets_by_object_key_invalid(self):
        
        result = self.runner.invoke(cli, [
            "list-assets", 
            "--object-key", "invalid-object-key"
        ])
        
        self.assertNotEqual(result.exit_code, 0)

    def test_list_assets_page_size(self):
        result = self.runner.invoke(cli, [
            "list-assets",
            "--id", self.asset_id,
            "--page-size", "10",
            "--page-offset", "0"
        ])
        self.assertEqual(result.exit_code, 0)
        output = json.loads(result.output)
        self.assertTrue(len(output["items"]) <= 10)
        
    def test_list_assets_invalid_page_size(self):
        result = self.runner.invoke(cli, [
            "list-assets",
            "--id", self.asset_id,
            "--page-size", "-1"
        ])
        self.assertNotEqual(result.exit_code, 0)

    def test_list_assets_page_offset(self):
        result = self.runner.invoke(cli, [
            "list-assets",
            "--id", self.asset_id,
            "--page-size", "1",
            "--page-offset", "0"
        ])
        self.assertEqual(result.exit_code, 0)

    def test_list_assets_invalid_offset(self):
        result = self.runner.invoke(cli, [
            "list-assets",
            "--id", self.asset_id,
            "--page-offset", "-1"
        ])
        self.assertNotEqual(result.exit_code, 0)
        
    def test_list_assets_page_offset_token(self):

        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.asset_id
        ])

        asset_details = json.loads(asset_details_result.output)
        parent_id = asset_details["properties"]["parentId"]        

        result = self.runner.invoke(cli, [
            "list-assets",
            "--id", parent_id,
            "--page-size", "1",
        ])
        
        result_json = json.loads(result.output)
        next_page_offset = result_json.get("nextPageOffset")

        if not next_page_offset:
            self.skipTest("No next page offset")
            
        result = self.runner.invoke(cli, [
            "list-assets",
            "--id", self.asset_id,
            "--page-offset-token", next_page_offset
        ])
        
        self.assertEqual(result.exit_code, 0)
        
    def test_list_asset_page_offset_token_invalid(self):
        result = self.runner.invoke(cli, [
            "list-assets",
            "--id", self.asset_id,
            "--page-offset-token", "invalid-token"
        ])
        self.assertNotEqual(result.exit_code, 0)
    
    def test_list_assets_sorting(self):
        result = self.runner.invoke(cli, [
            "list-assets",
            "--id", self.asset_id,
            "--order-by", "name",
            "--order-by-type", "desc"
        ])
        self.assertEqual(result.exit_code, 0)
        output = json.loads(result.output)
        # Verify sorting
        names = [item["name"] for item in output["items"]]
        self.assertEqual(names, sorted(names, reverse=True))
    
    def test_list_assets_missing_params(self):
        result = self.runner.invoke(cli, ["list-assets"])
        self.assertNotEqual(result.exit_code, 0)

class TestAssetDetails(TestAssetBase):
    """Tests for getting asset details"""

    def test_get_asset_details_by_id(self):
        """Test full asset details are returned"""
        result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.asset_id
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        try:
            output_json = json.loads(result.output)
            self.assertTrue(isinstance(output_json, dict))
        except json.JSONDecodeError:
            self.fail("Output is not valid JSON")
            
    def test_get_asset_details_by_id_invalid(self):
        """Test invalid ID returns an error"""
        result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", "invalid-id"
        ])
        
        self.assertNotEqual(result.exit_code, 0)
            
    def test_get_asset_details_by_url(self):
        """Test full asset details are returned"""
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.asset_id
        ])
        
        asset_details = json.loads(asset_details_result.output)
        url = asset_details["properties"]["url"]
        
        result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--url", url
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        try:
            output_json = json.loads(result.output)
            self.assertTrue(isinstance(output_json, dict))
        except json.JSONDecodeError:
            self.fail("Output is not valid JSON")
            
    def test_get_asset_details_by_url_invalid(self):
        """Test invalid URL returns an error"""
        result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--url", "invalid-url"
        ])
        
        self.assertNotEqual(result.exit_code, 0)
        
    def test_get_asset_details_by_object_key(self):
        """Test full asset details are returned"""
        self.setup_config()

        bucket = self.config.get("bucket")
        if not bucket:
            self.skipTest("No default bucket set")
            
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.asset_id
        ])
        
        asset_details = json.loads(asset_details_result.output)
        object_key = asset_details["properties"]["originalObjectKey"]
        
        result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--object-key", object_key
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        try:
            output_json = json.loads(result.output)
            self.assertTrue(isinstance(output_json, dict))
        except json.JSONDecodeError:
            self.fail("Output is not valid JSON")
            
    def test_get_asset_details_by_object_key_no_bucket(self):
        """Test missing bucket returns an error"""
        self.setup_config()

        bucket = self.config.get("bucket")
        if bucket:
            self.skipTest("Default bucket set")
            
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.asset_id
        ])
        
        asset_details = json.loads(asset_details_result.output)
        object_key = asset_details["properties"]["originalObjectKey"]
        
        result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--object-key", object_key
        ])
        
        self.assertNotEqual(result.exit_code, 0)
        
    def test_get_asset_details_by_object_key_invalid(self):
        """Test invalid object key returns an error"""
        result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--object-key", "invalid-object-key"
        ])
        
        self.assertNotEqual(result.exit_code, 0)
        
class TestAssetAddAssetProperties(TestAssetBase):
    """Tests for adding asset properties"""
    
    def test_add_asset_properties(self):
        """Test asset properties are added successfully"""
        result = self.runner.invoke(cli, [
            "add-asset-properties", 
            "--id", self.asset_id,
            "--properties", '{"test": "test"}'
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.asset_id
        ])
        
        asset_details = json.loads(asset_details_result.output)
        self.assertEqual(asset_details["customAttributes"]["test"], "test")
        
    def test_add_asset_properties_invalid_json(self):
        """Test invalid JSON returns an error"""
        result = self.runner.invoke(cli, [
            "add-asset-properties", 
            "--id", self.asset_id,
            "--properties", "invalid-json"
        ])
        
        self.assertNotEqual(result.exit_code, 0)
        
    def test_add_asset_properties_name(self):
        """Test asset properties are added successfully"""
        result = self.runner.invoke(cli, [
            "add-asset-properties", 
            "--id", self.asset_id,
            "--name", "test",
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.asset_id
        ])
        
        asset_details = json.loads(asset_details_result.output)
        self.assertEqual(asset_details["properties"]["displayName"], "test")
        
    def test_add_asset_properties_date(self):
        """Test asset properties are added successfully"""
        result = self.runner.invoke(cli, [
            "add-asset-properties", 
            "--id", self.asset_id,
            "--date", "2025-01-01T00:00:00Z",
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.asset_id
        ])
        
        asset_details = json.loads(asset_details_result.output)
        self.assertEqual(asset_details["properties"]["displayDate"], "2025-01-01T00:00:00Z")
        
    def test_add_asset_properties_invalid_date(self):
        """Test invalid date returns an error"""
        result = self.runner.invoke(cli, [
            "add-asset-properties", 
            "--id", self.asset_id,
            "--date", "invalid-date",
        ])
        
        self.assertNotEqual(result.exit_code, 0)
        
class TestAssetAddAssetCollection(TestAssetBase):
    """Tests for adding asset collections"""
    
    def test_add_asset_collection_with_id(self):
        """Test asset collection is added successfully"""
        result = self.runner.invoke(cli, [
            "add-asset-collection", 
            "--id", self.asset_id,
            "--collection-name", "test-collection"
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.asset_id
        ])
        
        asset_details = json.loads(asset_details_result.output)
        collections = asset_details["collections"]
        self.assertTrue(any(collection["description"] == "test-collection" for collection in collections))
        
    def test_add_asset_collection_with_id_invalid(self):
        """Test invalid ID returns an error"""
        result = self.runner.invoke(cli, [
            "add-asset-collection", 
            "--id", "invalid-id",
            "--collection-name", "test-collection"
        ])
        
        self.assertNotEqual(result.exit_code, 0)
        
    def test_add_asset_collection_with_url(self):
        """Test asset collection is added successfully"""
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.asset_id
        ])
        
        asset_details = json.loads(asset_details_result.output)
        url = asset_details["properties"]["url"]
        
        result = self.runner.invoke(cli, [
            "add-asset-collection", 
            "--url", url,
            "--collection-name", "test-collection1"
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--url", url
        ])
        
        asset_details = json.loads(asset_details_result.output)
        collections = asset_details["collections"]
        self.assertTrue(any(collection["description"] == "test-collection1" for collection in collections))
        
    def test_add_asset_collection_with_url_invalid(self):
        """Test invalid URL returns an error"""
        result = self.runner.invoke(cli, [
            "add-asset-collection", 
            "--url", "invalid-url",
            "--collection-name", "test-collection"
        ])
        
        self.assertNotEqual(result.exit_code, 0)

    def test_add_asset_collection_with_object_key(self):
        """Test asset collection is added successfully"""
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.asset_id
        ])
        
        asset_details = json.loads(asset_details_result.output)
        object_key = asset_details["properties"]["originalObjectKey"]
        
        result = self.runner.invoke(cli, [
            "add-asset-collection", 
            "--object-key", object_key,
            "--collection-name", "test-collection2"
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--object-key", object_key
        ])
        
        asset_details = json.loads(asset_details_result.output)
        collections = asset_details["collections"]
        self.assertTrue(any(collection["description"] == "test-collection2" for collection in collections))
        
    def test_add_asset_collection_with_object_key_no_bucket(self):
        """Test missing bucket returns an error"""
        self.setup_config()

        bucket = self.config.get("bucket")
        if bucket:
            self.skipTest("Default bucket set")

        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.asset_id
        ])

        asset_details = json.loads(asset_details_result.output)
        object_key = asset_details["properties"]["originalObjectKey"]

        result = self.runner.invoke(cli, [
            "add-asset-collection", 
            "--object-key", object_key,
            "--collection-name", "test-collection"
        ])

        self.assertNotEqual(result.exit_code, 0)

    def test_add_asset_collection_with_object_key_invalid(self):
        """Test invalid object key returns an error"""
        result = self.runner.invoke(cli, [
            "add-asset-collection", 
            "--object-key", "invalid-object-key",
            "--collection-name", "test-collection"
        ])
        
        self.assertNotEqual(result.exit_code, 0)

    def test_add_asset_collection_with_collection_id(self):
        """Test asset collection is added successfully"""
        collections = self.runner.invoke(cli, [
            "get-content-definition-contents",
            "--name", "Collection"
        ])

        collections = json.loads(collections.output)["items"]
        if len(collections) == 0:
            self.skipTest("No collections available")

        collection_id = collections[0]["id"]
        collection_name = collections[0]["title"]

        result = self.runner.invoke(cli, [
            "add-asset-collection", 
            "--id", self.asset_id,
            "--collection-id", collection_id
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.asset_id
        ])
        
        asset_details = json.loads(asset_details_result.output)
        collections = asset_details["collections"]
        self.assertTrue(any(collection["description"] == collection_name for collection in collections))
        
    def test_add_asset_collection_with_collection_id_invalid(self):
        """Test invalid collection ID returns an error"""
        result = self.runner.invoke(cli, [
            "add-asset-collection", 
            "--id", self.asset_id,
            "--collection-id", "invalid-id"
        ])
        
        self.assertNotEqual(result.exit_code, 0)
        
class TestAssetListAssetCollection(TestAssetBase):
    """Tests for listing asset collections"""
    
    def test_list_asset_tag_with_id(self):
        """Test asset collections are returned"""
        result = self.runner.invoke(cli, [
            "list-asset-collections", 
            "--id", self.asset_id
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        try:
            output_json = json.loads(result.output)
            self.assertTrue(isinstance(output_json, list))
        except json.JSONDecodeError:
            self.fail("Output is not valid JSON")

    def test_list_asset_tag_with_id_invalid(self):
        """Test invalid ID returns an error"""
        result = self.runner.invoke(cli, [
            "list-asset-collections", 
            "--id", "invalid-id"
        ])
        
        self.assertNotEqual(result.exit_code, 0)

    def test_list_asset_tag_with_url(self):
        """Test asset collections are returned"""
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.asset_id
        ])
        
        asset_details = json.loads(asset_details_result.output)
        url = asset_details["properties"]["url"]
        
        result = self.runner.invoke(cli, [
            "list-asset-collections", 
            "--url", url
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        try:
            output_json = json.loads(result.output)
            self.assertTrue(isinstance(output_json, list))
        except json.JSONDecodeError:
            self.fail("Output is not valid JSON")

    def test_list_asset_tag_with_url_invalid(self):
        """Test invalid URL returns an error"""
        result = self.runner.invoke(cli, [
            "list-asset-collections", 
            "--url", "invalid-url"
        ])
        
        self.assertNotEqual(result.exit_code, 0)

    def test_list_asset_tag_with_object_key(self):
        """Test asset collections are returned"""
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.asset_id
        ])
        
        asset_details = json.loads(asset_details_result.output)
        object_key = asset_details["properties"]["originalObjectKey"]
        
        result = self.runner.invoke(cli, [
            "list-asset-collections", 
            "--object-key", object_key
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        try:
            output_json = json.loads(result.output)
            self.assertTrue(isinstance(output_json, list))
        except json.JSONDecodeError:
            self.fail("Output is not valid JSON")

    def test_list_asset_tag_with_object_key_no_bucket(self):
        """Test missing bucket returns an error"""
        self.setup_config()

        bucket = self.config.get("bucket")
        if bucket:
            self.skipTest("Default bucket set")

        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.asset_id
        ])

        asset_details = json.loads(asset_details_result.output)
        object_key = asset_details["properties"]["originalObjectKey"]

        result = self.runner.invoke(cli, [
            "list-asset-collections", 
            "--object-key", object_key
        ])

        self.assertNotEqual(result.exit_code, 0)

    def test_list_asset_tag_with_object_key_invalid(self):
        """Test invalid object key returns an error"""
        result = self.runner.invoke(cli, [
            "list-asset-collections", 
            "--object-key", "invalid-object-key"
        ])
        
        self.assertNotEqual(result.exit_code, 0)

class TestAssetRemoveAssetCollection(TestAssetBase):
    """Tests for removing asset collections"""

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        
        collections = cls.runner.invoke(cli, [
            "get-content-definition-contents",
            "--name", "Collection"
        ])

        collections = json.loads(collections.output)["items"]
        if len(collections) == 0:
            cls.skipTest("No collections available")

        cls.collection_id = collections[0]["id"]
        cls.collection_name = collections[0]["title"]

    def test_remove_asset_collection_id(self):
        """Test asset collection is removed successfully"""
        result = self.runner.invoke(cli, [
            "add-asset-collection", 
            "--id", self.asset_id,
            "--collection-id", self.collection_id
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        result = self.runner.invoke(cli, [
            "remove-asset-collection", 
            "--id", self.asset_id,
            "--collection-id", self.collection_id
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.asset_id
        ])
        
        asset_details = json.loads(asset_details_result.output)
        collections = asset_details["collections"]
        self.assertFalse(self.collection_name in collections)
        
    def test_remove_asset_collection_id_invalid(self):
        """Test invalid collection ID returns an error"""
        result = self.runner.invoke(cli, [
            "remove-asset-collection", 
            "--id", self.asset_id,
            "--collection-id", "invalid-id"
        ])
        
        self.assertNotEqual(result.exit_code, 0)
        
    def test_remove_asset_collection_with_url(self):
        """Test asset collection is removed successfully"""
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.asset_id
        ])
        
        asset_details = json.loads(asset_details_result.output)
        url = asset_details["properties"]["url"]
        
        result = self.runner.invoke(cli, [
            "add-asset-collection", 
            "--url", url,
            "--collection-id", self.collection_id
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        result = self.runner.invoke(cli, [
            "remove-asset-collection", 
            "--url", url,
            "--collection-id", self.collection_id
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--url", url
        ])
        
        asset_details = json.loads(asset_details_result.output)
        collections = asset_details["collections"]
        self.assertFalse(self.collection_name in collections)
        
    def test_remove_asset_collection_with_url_invalid(self):
        """Test invalid URL returns an error"""
        result = self.runner.invoke(cli, [
            "remove-asset-collection", 
            "--url", "invalid-url",
            "--collection-name", "test-collection"
        ])
        
        self.assertNotEqual(result.exit_code, 0)
        
    def test_remove_asset_collection_with_object_key(self):
        """Test asset collection is removed successfully"""
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.asset_id
        ])
        
        asset_details = json.loads(asset_details_result.output)
        object_key = asset_details["properties"]["originalObjectKey"]
        
        result = self.runner.invoke(cli, [
            "add-asset-collection", 
            "--object-key", object_key,
            "--collection-id", self.collection_id
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        result = self.runner.invoke(cli, [
            "remove-asset-collection", 
            "--object-key", object_key,
            "--collection-id", self.collection_id
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--object-key", object_key
        ])
        
        asset_details = json.loads(asset_details_result.output)
        collections = asset_details["collections"]
        self.assertFalse(self.collection_name in collections)
        
    def test_remove_asset_collection_with_object_key_no_bucket(self):
        """Test missing bucket returns an error"""
        self.setup_config()

        bucket = self.config.get("bucket")
        if bucket:
            self.skipTest("Default bucket set")

        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.asset_id
        ])

        asset_details = json.loads(asset_details_result.output)
        object_key = asset_details["properties"]["originalObjectKey"]

        result = self.runner.invoke(cli, [
            "add-asset-collection", 
            "--object-key", object_key,
            "--collection-id", self.collection_id
        ])

        self.assertNotEqual(result.exit_code, 0)

    def test_remove_asset_collection_with_object_key_invalid(self):
        """Test invalid object key returns an error"""
        result = self.runner.invoke(cli, [
            "remove-asset-collection", 
            "--object-key", "invalid-object-key",
            "--collection-name", "test-collection"
        ])
        
        self.assertNotEqual(result.exit_code, 0)

class TestAssetAddAssetTag(TestAssetBase):
    """Tests for adding asset tags"""

    def test_add_asset_tag_with_id(self):
        """Test asset tag is added successfully"""
        result = self.runner.invoke(cli, [
            "add-asset-tag", 
            "--id", self.asset_id,
            "--tag-name", "test-tag"
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.asset_id
        ])
        
        asset_details = json.loads(asset_details_result.output)
        tags = asset_details["tags"]
        self.assertTrue(any(tag["description"] == "test-tag" for tag in tags))
        
    def test_add_asset_tag_with_id_invalid(self):
        """Test invalid ID returns an error"""
        result = self.runner.invoke(cli, [
            "add-asset-tag", 
            "--id", "invalid-id",
            "--tag-name", "test-tag"
        ])
        
        self.assertNotEqual(result.exit_code, 0)
        
    def test_add_asset_tag_with_url(self):
        """Test asset tag is added successfully"""
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.asset_id
        ])
        
        asset_details = json.loads(asset_details_result.output)
        url = asset_details["properties"]["url"]
        
        result = self.runner.invoke(cli, [
            "add-asset-tag", 
            "--url", url,
            "--tag-name", "test-tag1"
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--url", url
        ])
        
        asset_details = json.loads(asset_details_result.output)
        tags = asset_details["tags"]
        self.assertTrue(any(tag["description"] == "test-tag1" for tag in tags))
        
    def test_add_asset_tag_with_url_invalid(self):
        """Test invalid URL returns an error"""
        result = self.runner.invoke(cli, [
            "add-asset-tag", 
            "--url", "invalid-url",
            "--tag-name", "test-tag"
        ])
        
        self.assertNotEqual(result.exit_code, 0)

    def test_add_asset_tag_with_object_key(self):
        """Test asset tag is added successfully"""
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.asset_id
        ])
        
        asset_details = json.loads(asset_details_result.output)
        object_key = asset_details["properties"]["originalObjectKey"]
        
        result = self.runner.invoke(cli, [
            "add-asset-tag", 
            "--object-key", object_key,
            "--tag-name", "test-tag2"
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--object-key", object_key
        ])
        
        asset_details = json.loads(asset_details_result.output)
        tags = asset_details["tags"]
        self.assertTrue(any(tag["description"] == "test-tag2" for tag in tags))
        
    def test_add_asset_tag_with_object_key_no_bucket(self):
        """Test missing bucket returns an error"""
        self.setup_config()

        bucket = self.config.get("bucket")
        if bucket:
            self.skipTest("Default bucket set")

        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.asset_id
        ])

        asset_details = json.loads(asset_details_result.output)
        object_key = asset_details["properties"]["originalObjectKey"]

        result = self.runner.invoke(cli, [
            "add-asset-tag", 
            "--object-key", object_key,
            "--tag-name", "test-tag"
        ])

        self.assertNotEqual(result.exit_code, 0)

    def test_add_asset_tag_with_object_key_invalid(self):
        """Test invalid object key returns an error"""
        result = self.runner.invoke(cli, [
            "add-asset-tag", 
            "--object-key", "invalid-object-key",
            "--tag-name", "test-tag"
        ])
        
        self.assertNotEqual(result.exit_code, 0)

    def test_add_asset_tag_with_tag_id(self):
        """Test asset tag is added successfully"""
        tags = self.runner.invoke(cli, [
            "get-content-definition-contents",
            "--name", "Tag"
        ])

        tags = json.loads(tags.output)["items"]
        if len(tags) == 0:
            self.skipTest("No tags available")

        tag_id = tags[0]["id"]
        tag_name = tags[0]["title"]

        result = self.runner.invoke(cli, [
            "add-asset-tag", 
            "--id", self.asset_id,
            "--tag-id", tag_id
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.asset_id
        ])
        
        asset_details = json.loads(asset_details_result.output)
        tags = asset_details["tags"]
        self.assertTrue(any(tag["description"] == tag_name for tag in tags))

    def test_add_asset_tag_with_tag_id_invalid(self):
        """Test invalid tag ID returns an error"""
        result = self.runner.invoke(cli, [
            "add-asset-tag", 
            "--id", self.asset_id,
            "--tag-id", "invalid-id"
        ])
        
        self.assertNotEqual(result.exit_code, 0)

class TestAssetListAssetTag(TestAssetBase):
    """Tests for listing asset tags"""

    def test_list_asset_tag_with_id(self):
        """Test asset tags are returned"""
        result = self.runner.invoke(cli, [
            "list-asset-tags", 
            "--id", self.asset_id
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        try:
            output_json = json.loads(result.output)
            self.assertTrue(isinstance(output_json, list))
        except json.JSONDecodeError:
            self.fail("Output is not valid JSON")

    def test_list_asset_tag_with_id_invalid(self):
        """Test invalid ID returns an error"""
        result = self.runner.invoke(cli, [
            "list-asset-tags", 
            "--id", "invalid-id"
        ])
        
        self.assertNotEqual(result.exit_code, 0)

    def test_list_asset_tag_with_url(self):
        """Test asset tags are returned"""
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.asset_id
        ])
        
        asset_details = json.loads(asset_details_result.output)
        url = asset_details["properties"]["url"]
        
        result = self.runner.invoke(cli, [
            "list-asset-tags", 
            "--url", url
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        try:
            output_json = json.loads(result.output)
            self.assertTrue(isinstance(output_json, list))
        except json.JSONDecodeError:
            self.fail("Output is not valid JSON")

    def test_list_asset_tag_with_url_invalid(self):
        """Test invalid URL returns an error"""
        result = self.runner.invoke(cli, [
            "list-asset-tags", 
            "--url", "invalid-url"
        ])
        
        self.assertNotEqual(result.exit_code, 0)

    def test_list_asset_tag_with_object_key(self):
        """Test asset tags are returned"""
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.asset_id
        ])
        
        asset_details = json.loads(asset_details_result.output)
        object_key = asset_details["properties"]["originalObjectKey"]
        
        result = self.runner.invoke(cli, [
            "list-asset-tags", 
            "--object-key", object_key
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        try:
            output_json = json.loads(result.output)
            self.assertTrue(isinstance(output_json, list))
        except json.JSONDecodeError:
            self.fail("Output is not valid JSON")

    def test_list_asset_tag_with_object_key_no_bucket(self):
        """Test missing bucket returns an error"""
        self.setup_config()

        bucket = self.config.get("bucket")
        if bucket:
            self.skipTest("Default bucket set")

        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.asset_id
        ])

        asset_details = json.loads(asset_details_result.output)
        object_key = asset_details["properties"]["originalObjectKey"]

        result = self.runner.invoke(cli, [
            "list-asset-tags", 
            "--object-key", object_key
        ])

        self.assertNotEqual(result.exit_code, 0)

    def test_list_asset_tag_with_object_key_invalid(self):
        """Test invalid object key returns an error"""
        result = self.runner.invoke(cli, [
            "list-asset-tags", 
            "--object-key", "invalid-object-key"
        ])
        
        self.assertNotEqual(result.exit_code, 0)

class TestAssetRemoveAssetTag(TestAssetBase):
    """Tests for removing asset tags"""

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        
        tags = cls.runner.invoke(cli, [
            "get-content-definition-contents",
            "--name", "Tag"
        ])

        tags = json.loads(tags.output)["items"]
        if len(tags) == 0:
            cls.skipTest("No tags available")

        cls.tag_id = tags[0]["id"]
        cls.tag_name = tags[0]["title"]
        
        tag_result = cls.runner.invoke(cli, [
            "add-asset-tag", 
            "--id", cls.asset_id,
            "--tag-id", cls.tag_id
        ])
        
        if tag_result.exit_code != 0:
            cls.skipTest("Failed to add tag")

    def test_remove_asset_tag_id(self):
        """Test asset tag is removed successfully"""
        result = self.runner.invoke(cli, [
            "add-asset-tag", 
            "--id", self.asset_id,
            "--tag-id", self.tag_id
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        result = self.runner.invoke(cli, [
            "remove-asset-tag", 
            "--id", self.asset_id,
            "--tag-id", self.tag_id
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.asset_id
        ])
        
        asset_details = json.loads(asset_details_result.output)
        tags = asset_details["tags"]
        self.assertFalse(self.tag_name in tags)
        
    def test_remove_asset_tag_id_invalid(self):
        """Test invalid tag ID returns an error"""
        result = self.runner.invoke(cli, [
            "remove-asset-tag", 
            "--id", self.asset_id,
            "--tag-id", "invalid-id"
        ])
        
        self.assertNotEqual(result.exit_code, 0)
        
    def test_remove_asset_tag_with_url(self):
        """Test asset tag is removed successfully"""
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.asset_id
        ])
        
        asset_details = json.loads(asset_details_result.output)
        url = asset_details["properties"]["url"]
        
        result = self.runner.invoke(cli, [
            "add-asset-tag", 
            "--url", url,
            "--tag-id", self.tag_id
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        result = self.runner.invoke(cli, [
            "remove-asset-tag", 
            "--url", url,
            "--tag-id", self.tag_id
        ])
        
        self.assertEqual(result.exit_code, 0)

        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--url", url
        ])

        asset_details = json.loads(asset_details_result.output)
        tags = asset_details["tags"]
        self.assertFalse(self.tag_name in tags)

    def test_remove_asset_tag_with_url_invalid(self):
        """Test invalid URL returns an error"""
        result = self.runner.invoke(cli, [
            "remove-asset-tag", 
            "--url", "invalid-url",
            "--tag-id", self.tag_id
        ])
        
        self.assertNotEqual(result.exit_code, 0)

    def test_remove_asset_tag_with_object_key(self):
        """Test asset tag is removed successfully"""
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.asset_id
        ])
        
        asset_details = json.loads(asset_details_result.output)
        object_key = asset_details["properties"]["originalObjectKey"]
        
        result = self.runner.invoke(cli, [
            "add-asset-tag", 
            "--object-key", object_key,
            "--tag-id", self.tag_id
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        result = self.runner.invoke(cli, [
            "remove-asset-tag", 
            "--object-key", object_key,
            "--tag-id", self.tag_id
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--object-key", object_key
        ])
        
        asset_details = json.loads(asset_details_result.output)
        tags = asset_details["tags"]
        self.assertFalse(self.tag_name in tags)

    def test_remove_asset_tag_with_object_key_no_bucket(self):
        """Test missing bucket returns an error"""
        self.setup_config()

        bucket = self.config.get("bucket")
        if bucket:
            self.skipTest("Default bucket set")

        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.asset_id
        ])

        asset_details = json.loads(asset_details_result.output)
        object_key = asset_details["properties"]["originalObjectKey"]

        result = self.runner.invoke(cli, [
            "add-asset-tag", 
            "--object-key", object_key,
            "--tag-id", self.tag_id
        ])

        self.assertNotEqual(result.exit_code, 0)

    def test_remove_asset_tag_with_object_key_invalid(self):
        """Test invalid object key returns an error"""
        result = self.runner.invoke(cli, [
            "remove-asset-tag", 
            "--object-key", "invalid-object-key",
            "--tag-id", self.tag_id
        ])
        
        self.assertNotEqual(result.exit_code, 0)

class TestAssetAddRelatedContent(TestAssetBase):
    """Tests for adding related content"""

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        
        series_result = cls.runner.invoke(cli, [
            "get-content-definition-contents",
            "--name", "Series"
        ])

        countries = json.loads(series_result.output)["items"]
        if len(countries) == 0:
            cls.skipTest("Content definition not available")

        cls.series_id = countries[0]["id"]
        cls.series_name = countries[0]["title"]

    def test_add_related_content_with_id(self):
        """Test related content is added successfully"""
        result = self.runner.invoke(cli, [
            "add-asset-related-content", 
            "--id", self.asset_id,
            "--related-content-id", self.series_id
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.asset_id
        ])
        
        asset_details = json.loads(asset_details_result.output)
        related_contents = asset_details["relatedContent"]
        self.assertTrue(any(content["id"] == self.series_id for content in related_contents))
        
    def test_add_related_content_with_id_invalid(self):
        """Test invalid ID returns an error"""
        result = self.runner.invoke(cli, [
            "add-asset-related-content", 
            "--id", "invalid-id",
            "--related-content-id", self.series_id
        ])
        
        self.assertNotEqual(result.exit_code, 0)
        
    def test_add_related_content_with_url(self):
        """Test related content is added successfully"""
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.asset_id
        ])
        
        asset_details = json.loads(asset_details_result.output)
        url = asset_details["properties"]["url"]
        
        result = self.runner.invoke(cli, [
            "add-asset-related-content", 
            "--url", url,
            "--related-content-id", self.series_id
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--url", url
        ])
        
        asset_details = json.loads(asset_details_result.output)
        related_contents = asset_details["relatedContent"]

        self.assertTrue(any(content["id"] == self.series_id for content in related_contents))

    def test_add_related_content_with_url_invalid(self):
        """Test invalid URL returns an error"""
        result = self.runner.invoke(cli, [
            "add-asset-related-content", 
            "--url", "invalid-url",
            "--related-content-id", self.series_id
        ])
        
        self.assertNotEqual(result.exit_code, 0)

    def test_add_related_content_with_object_key(self):
        """Test related content is added successfully"""
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.asset_id
        ])
        
        asset_details = json.loads(asset_details_result.output)
        object_key = asset_details["properties"]["originalObjectKey"]
        
        result = self.runner.invoke(cli, [
            "add-asset-related-content", 
            "--object-key", object_key,
            "--related-content-id", self.series_id
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--object-key", object_key
        ])
        
        asset_details = json.loads(asset_details_result.output)
        related_contents = asset_details["relatedContent"]
        self.assertTrue(any(content["id"] == self.series_id for content in related_contents))

    def test_add_related_content_with_object_key_no_bucket(self):
        """Test missing bucket returns an error"""
        self.setup_config()

        bucket = self.config.get("bucket")
        if bucket:
            self.skipTest("Default bucket set")

        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.asset_id
        ])

        asset_details = json.loads(asset_details_result.output)
        object_key = asset_details["properties"]["originalObjectKey"]

        result = self.runner.invoke(cli, [
            "add-asset-related-content", 
            "--object-key", object_key,
            "--related-content-id", self.series_id
        ])

        self.assertNotEqual(result.exit_code, 0)

    def test_add_related_content_with_object_key_invalid(self):
        """Test invalid object key returns an error"""
        result = self.runner.invoke(cli, [
            "add-asset-related-content", 
            "--object-key", "invalid-object-key",
            "--related-content-id", self.series_id
        ])
        
        self.assertNotEqual(result.exit_code, 0)

    def test_add_related_content_with_content_id_invalid(self):
        """Test invalid content ID returns an error"""
        result = self.runner.invoke(cli, [
            "add-asset-related-content", 
            "--id", self.asset_id,
            "--related-content-id", "invalid-id"
        ])
        
        self.assertNotEqual(result.exit_code, 0)
        
class TestDeleteAsset(TestAssetBase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        
        with open("nomad_media_cli/tests/test-config.json", "r") as file:
            test_config = json.load(file)
            cls.test_dir_id = test_config["testDirId"]
        
        result = cls.runner.invoke(cli, [
            "upload-assets", 
            "--source", "setup.py",
            "--id", cls.test_dir_id
        ])
        
        if result.exit_code != 0:
            raise Exception(f"Failed to upload asset: {result.output}")
        
        cls.delete_asset_id = result.output.replace('"', "").strip()

    @classmethod
    def tearDownClass(cls):
        result = cls.runner.invoke(cli, [
            "delete-asset", 
            "--id", cls.delete_asset_id
        ])
        
        if result.exit_code != 0:
            raise Exception(f"Failed to delete asset: {result.output}")

    def test_delete_asset_with_id(self):
        """Test asset is deleted successfully"""
        result = self.runner.invoke(cli, [
            "delete-asset", 
            "--id", self.delete_asset_id
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.delete_asset_id
        ])
        
        self.assertNotEqual(asset_details_result.exit_code, 0)

    def test_delete_asset_with_id_invalid(self):
        """Test invalid ID returns an error"""
        result = self.runner.invoke(cli, [
            "delete-asset", 
            "--id", "invalid-id"
        ])
        
        self.assertNotEqual(result.exit_code, 0)

    def test_delete_asset_with_url(self):
        """Test asset is deleted successfully"""
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.delete_asset_id
        ])
        
        asset_details = json.loads(asset_details_result.output)
        url = asset_details["properties"]["url"]
        
        result = self.runner.invoke(cli, [
            "delete-asset", 
            "--url", url
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--url", url
        ])
        
        self.assertNotEqual(asset_details_result.exit_code, 0)

    def test_delete_asset_with_url_invalid(self):
        """Test invalid URL returns an error"""
        result = self.runner.invoke(cli, [
            "delete-asset", 
            "--url", "invalid-url"
        ])
        
        self.assertNotEqual(result.exit_code, 0)

    def test_delete_asset_with_object_key(self):
        """Test asset is deleted successfully"""
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.delete_asset_id
        ])
        
        asset_details = json.loads(asset_details_result.output)
        object_key = asset_details["properties"]["originalObjectKey"]
        
        result = self.runner.invoke(cli, [
            "delete-asset", 
            "--object-key", object_key
        ])
        
        self.assertEqual(result.exit_code, 0)
        
        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--object-key", object_key
        ])
        
        self.assertNotEqual(asset_details_result.exit_code, 0)

    def test_delete_asset_with_object_key_no_bucket(self):
        """Test missing bucket returns an error"""
        self.setup_config()

        bucket = self.config.get("bucket")
        if bucket:
            self.skipTest("Default bucket set")

        asset_details_result = self.runner.invoke(cli, [
            "get-asset-details", 
            "--id", self.delete_asset_id
        ])

        asset_details = json.loads(asset_details_result.output)
        object_key = asset_details["properties"]["originalObjectKey"]

        result = self.runner.invoke(cli, [
            "delete-asset", 
            "--object-key", object_key
        ])

        self.assertNotEqual(result.exit_code, 0)

    def test_delete_asset_with_object_key_invalid(self):
        """Test invalid object key returns an error"""
        result = self.runner.invoke(cli, [
            "delete-asset", 
            "--object-key", "invalid-object-key"
        ])
        
        self.assertNotEqual(result.exit_code, 0)

if __name__ == "__main__":
    unittest.main()
        



from nomad_media_cli.helpers.utils import initialize_sdk
import click
import json
import sys

@click.command()
@click.option("--content-id", required=True, help="The ID of the content.")
@click.option("--content-definition-id", required=True, help="The ID of the content definition.")
@click.option("--sort_column", default=None, help="The sort column.")
@click.option("--is-desc", default=None, type=click.BOOL, help="The is descending flag.")
@click.option("--page-index", default=None, type=click.INT, help="The page index.")
@click.option("--size-index", default=None, type=click.INT, help="The size index.")
@click.pass_context
def get_content_user_track(ctx, content_id, content_definition_id, sort_column, is_desc, page_index, size_index):
    """Get the user track for a content"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        result = nomad_sdk.get_content_user_track(content_id, content_definition_id, sort_column, is_desc, page_index, size_index)
        click.echo(json.dumps(result, indent=4))

    except Exception as e:
        click.echo(json.dumps({ "error": f"Error getting content user track: {e}" }))
        sys.exit(1)


import click
import json
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--live-operator-id", required=True, help="The ID of the live operator.")
@click.option("--preroll-asset-id", required=False, help="The preroll asset ID of the live operator.")
@click.option("--postroll-asset-id", required=False, help="The postroll asset ID of the live operator.")
@click.option("--live-input-id", required=False, help="The live input ID of the live operator.")
@click.option("--related-content-ids", required=False, help="The related content IDs of the live operator in JSON list format.")
@click.option("--tag-ids", required=False, help="The tag IDs of the live operator in JSON list format.")
@click.pass_context
def start_broadcast(ctx, live_operator_id, preroll_asset_id, postroll_asset_id, live_input_id, related_content_ids, tag_ids):
    """Start broadcast"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        nomad_sdk.start_broadcast(
            live_operator_id,
            preroll_asset_id,
            postroll_asset_id,
            live_input_id,
            json.loads(related_content_ids) if related_content_ids else None,
            json.loads(tag_ids) if tag_ids else None
        )
        click.echo("Broadcast started successfully.")

    except Exception as e:
        click.echo(json.dumps({"error": f"Error starting broadcast: {e}"}))
        sys.exit(1)


import click
import json
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--schedule-id", required=True, help="The ID of the schedule the schedule item live channel is to be updated from.")
@click.option("--item-id", required=True, help="The ID of the item to be updated.")
@click.option("--days", required=False, help="The days of the schedule item live channel in JSON list format.")
@click.option("--duration-time-code", required=False, help="The duration time between time_code and end_time_code. Format: hh:mm:ss;ff.")
@click.option("--end-time-code", required=False, help="The end time code of the schedule item live channel. Format: hh:mm:ss;ff.")
@click.option("--live-channel", required=False, help="The live channel of the schedule item live channel in JSON dict format.")
@click.option("--time-code", required=False, help="The time code of the schedule item live channel. Format: hh:mm:ss;ff.")
@click.pass_context
def update_schedule_item_live_channel(ctx, schedule_id, item_id, days, duration_time_code, end_time_code, live_channel, time_code):
    """Update schedule item live channel"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        result = nomad_sdk.update_schedule_item_live_channel(
            schedule_id,
            item_id,
            json.loads(days) if days else None,
            duration_time_code,
            end_time_code,
            json.loads(live_channel) if live_channel else None,
            time_code
        )
        click.echo("Schedule item live channel updated successfully.")
        click.echo(json.dumps(result, indent=4))

    except Exception as e:
        click.echo(json.dumps({"error": f"Error updating schedule item live channel: {e}"}))
        sys.exit(1)


import click
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option('--query', default=None, help='Search query')
@click.option('--offset', default=0, help='Page offset')
@click.option('--size', default=100, help='Page size')
@click.option('--filters', default=None, help='Filters')
@click.option('--sort-fields', default=None, help='Sort fields')
@click.option('--search-result-fields', default=None, help='Search result fields')
@click.option('--full-url-field-names', default=None, help='Full URL field names')
@click.option('--distinct-on-field-name', default=None, help='Distinct on field name')
@click.option('--include-video-clips', is_flag=True, help='Include video clips')
@click.option('--similar-asset-id', default=None, help='Similar asset ID')
@click.option('--min-score', default=None, help='Minimum score')
@click.option('--exclude-total-record-count', is_flag=True, help='Exclude total record count')
@click.option('--filter-binder', default=None, help='Filter binder')
@click.option('--use-llm-search', is_flag=True, help='Use LLM search')
@click.option('--include-internal-fields-in-results', is_flag=True, help='Include internal fields in results')
def search_cli(query, offset, size, filters, sort_fields, search_result_fields, full_url_field_names, distinct_on_field_name, include_video_clips, similar_asset_id, min_score, exclude_total_record_count, filter_binder, use_llm_search, include_internal_fields_in_results):
    """CLI for search function."""
    initialize_sdk()
    nomad_sdk = click.get_current_context().obj["nomad_sdk"]
    
    result = nomad_sdk.search(
        query=query,
        offset=offset,
        size=size,
        filters=filters,
        sort_fields=sort_fields,
        search_result_fields=search_result_fields,
        full_url_field_names=full_url_field_names,
        distinct_on_field_name=distinct_on_field_name,
        include_video_clips=include_video_clips,
        similar_asset_id=similar_asset_id,
        min_score=min_score,
        exclude_total_record_count=exclude_total_record_count,
        filter_binder=filter_binder,
        use_llm_search=use_llm_search,
        include_internal_fields_in_results=include_internal_fields_in_results
    )
    click.echo(result)

if __name__ == '__main__':
    search_cli()


import click
import json
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--event-id", required=True, help="The ID of the event to add the live schedule to.")
@click.option("--slate-video", required=False, help="The slate video ID of the event in JSON dict format.")
@click.option("--preroll-video", required=False, help="The preroll video of the event in JSON dict format.")
@click.option("--postroll-video", required=False, help="The postroll video of the event in JSON dict format.")
@click.option("--is-secure-output", required=False, type=click.BOOL, help="Whether the event is secure output.")
@click.option("--archive-folder", required=False, help="The archive folder of the event in JSON dict format.")
@click.option("--primary-live-input", required=False, help="The live input A ID of the event in JSON dict format.")
@click.option("--backup-live-input", required=False, help="The live input B ID of the event in JSON dict format.")
@click.option("--primary-livestream-input-url", required=False, help="The primary live stream URL of the event.")
@click.option("--backup-livestream-input-url", required=False, help="The backup live stream URL of the event.")
@click.option("--external-output-profiles", required=False, help="The external output profiles of the event in JSON list format.")
@click.option("--status", required=False, help="Current status of the Live Channel Settings configuration in JSON dict format.")
@click.option("--status-message", required=False, help="The status message of the event.")
@click.option("--live-channel", required=False, help="The live channel of the event in JSON dict format.")
@click.option("--override-settings", required=False, type=click.BOOL, help="Whether to override the settings of the event.")
@click.option("--output-profile-group", required=False, help="The output profile group of the event in JSON dict format.")
@click.pass_context
def add_live_schedule_to_event(ctx, event_id, slate_video, preroll_video, postroll_video, is_secure_output, archive_folder, primary_live_input, backup_live_input, primary_livestream_input_url, backup_livestream_input_url, external_output_profiles, status, status_message, live_channel, override_settings, output_profile_group):
    """Add live schedule to event"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        result = nomad_sdk.add_live_schedule_to_event(
            event_id,
            json.loads(slate_video) if slate_video else None,
            json.loads(preroll_video) if preroll_video else None,
            json.loads(postroll_video) if postroll_video else None,
            is_secure_output,
            json.loads(archive_folder) if archive_folder else None,
            json.loads(primary_live_input) if primary_live_input else None,
            json.loads(backup_live_input) if backup_live_input else None,
            primary_livestream_input_url,
            backup_livestream_input_url,
            json.loads(external_output_profiles) if external_output_profiles else None,
            json.loads(status) if status else None,
            status_message,
            json.loads(live_channel) if live_channel else None,
            override_settings,
            json.loads(output_profile_group) if output_profile_group else None
        )
        click.echo("Live schedule added to event successfully.")

    except Exception as e:
        click.echo(json.dumps({"error": f"Error adding live schedule to event: {e}"}))
        sys.exit(1)


import click
import json
import sys

def get_id_from_url(ctx, url, object_key, nomad_sdk):
    if url and "::" not in url:
        click.echo(json.dumps({ "error": "Please provide a valid path or set the default bucket." }))               
        sys.exit(1)
    elif object_key:
        if "bucket" in ctx.obj["config"]:
            url = f"{ctx.obj["config"]["bucket"]}::{object_key}"
        else:
            click.echo(json.dumps({ "error": "Please set bucket using `set-bucket` or use url." }))
            sys.exit(1)

    url_search_results = nomad_sdk.search(None, None, None, [{
        "fieldName": "url",
        "operator": "equals",
        "values": url
    }], None, None, None, None, None, None, None, None, None, None, None)
    
    if not url_search_results or len(url_search_results["items"]) == 0:
        click.echo(json.dumps({ "error": f"URL {url} not found." }))
        sys.exit(1)
        
    return url_search_results["items"][0]["id"]


import click
import json
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--id", required=True, help="The ID of the asset to local restore.")
@click.option("--profile", required=False, help="The profile of the local restore.")
@click.pass_context
def local_restore_asset(ctx, id, profile):
    """Local restore asset"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        result = nomad_sdk.local_restore_asset(id, profile)
        click.echo("Asset local restored successfully.")
        click.echo(json.dumps(result, indent=4))

    except Exception as e:
        click.echo(json.dumps({"error": f"Error local restoring asset: {e}"}))
        sys.exit(1)


import click
import json
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--name", required=False, help="The name of the live input.")
@click.option("--source", required=False, help="The source of the live input.")
@click.option("--input-type", required=False, help="The type of the live input. The types are RTMP_PULL, RTMP_PUSH, RTP_PUSH, UDP_PUSH and URL_PULL.")
@click.option("--is-standard", required=False, type=click.BOOL, help="Indicates if the live input is standard.")
@click.option("--video-asset-id", required=False, help="The video asset ID of the live input.")
@click.option("--destinations", required=True, help="The destinations of the live input in JSON list format.")
@click.option("--sources", required=False, help="The sources of the live input in JSON list format.")
@click.pass_context
def create_live_input(ctx, name, source, input_type, is_standard, video_asset_id, destinations, sources):
    """Create live input"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        result = nomad_sdk.create_live_input(
            name,
            source,
            input_type,
            is_standard,
            video_asset_id,
            json.loads(destinations),
            json.loads(sources) if sources else None
        )
        click.echo(json.dumps(result, indent=4))

    except Exception as e:
        click.echo(json.dumps({"error": f"Error creating live input: {e}"}))
        sys.exit(1)


import click
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--code", required=True, help="The code of the user.")
@click.option("--new-password", required=True, help="The new password of the user.")
@click.pass_context
def reset_password(ctx, code, new_password):
    """Reset password"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        nomad_sdk.reset_password(code, new_password)
        click.echo("Password reset successfully.")

    except Exception as e:
        click.echo(json.dumps({"error": f"Error resetting password: {e}"}))
        sys.exit(1)


import click
import json
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--name", required=True, help="The name of the playlist.")
@click.option("--thumbnail-asset", required=False, help="The thumbnail asset of the playlist in JSON dict format.")
@click.option("--loop-playlist", required=True, type=click.BOOL, help="Whether the playlist is looped.")
@click.option("--default-video-asset", required=True, help="The default video asset of the playlist in JSON dict format.")
@click.pass_context
def create_playlist(ctx, name, thumbnail_asset, loop_playlist, default_video_asset):
    """Create playlist"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        result = nomad_sdk.create_playlist(
            name,
            json.loads(thumbnail_asset) if thumbnail_asset else None,
            loop_playlist,
            json.loads(default_video_asset)
        )
        click.echo(json.dumps(result, indent=4))

    except Exception as e:
        click.echo(json.dumps({"error": f"Error creating playlist: {e}"}))
        sys.exit(1)


import click
import json
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--id", required=True, help="The ID of the asset to update the ad break for.")
@click.option("--ad-break-id", required=True, help="The ID of the ad break.")
@click.option("--time-code", required=False, help="The time code of the asset ad break. Format: hh:mm:ss;ff.")
@click.option("--tags", required=False, help="The tags of the asset ad break in JSON list format.")
@click.option("--labels", required=False, help="The labels of the asset ad break in JSON list format.")
@click.pass_context
def update_asset_ad_break(ctx, id, ad_break_id, time_code, tags, labels):
    """Update asset ad break"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        result = nomad_sdk.update_asset_ad_break(
            id,
            ad_break_id,
            time_code,
            json.loads(tags) if tags else None,
            json.loads(labels) if labels else None
        )
        click.echo("Asset ad break updated successfully.")
        click.echo(json.dumps(result, indent=4))

    except Exception as e:
        click.echo(json.dumps({"error": f"Error updating asset ad break: {e}"}))
        sys.exit(1)


import click
import json
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--id", required=True, help="The asset ID of the current item to get the parents for.")
@click.option("--page-size", required=True, type=click.INT, help="The size of the page of folders to retrieve.")
@click.pass_context
def get_asset_parent_folders(ctx, id, page_size):
    """Get asset parent folders"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        result = nomad_sdk.get_asset_parent_folders(id, page_size)
        click.echo(json.dumps(result, indent=4))

    except Exception as e:
        click.echo(json.dumps({"error": f"Error getting asset parent folders: {e}"}))
        sys.exit(1)


from nomad_media_cli.helpers.utils import initialize_sdk, get_config

import click
import json
import sys

@click.command()
@click.option("--id", help="Asset ID, collection id, or saved search id to list the assets for")
@click.option("--url", help="The Nomad URL of the Asset (file or folder) to list the assets for (bucket::object-key).")
@click.option("--object-key", help="Object-key of the Asset (file or folder) to list the assets for. This option assumes the default bucket that was previously set with the `set-bucket` command.")
@click.option("--page-size", default=100, type=click.INT, help="Number of items to return per page. Default is 100.")
@click.option("--page-offset", default=0, type=click.INT, help="Offset of the page. Default is 0. The recommendation is to use page-token instead of page-offset.")
@click.option("--page-offset-token", help="Token to get the next page. Cannot be used with page-offset. This value is returned in a previous list-assets API call.")
@click.option("--order-by", default="title", help="Field to order by. Default is title.")
@click.option("--order-by-type", default="ascending", type=click.Choice(["asc", "desc", "ascending", "descending"]), help="Order type (asc, desc, ascending, descending). Default is ascending.")
@click.option("-r", "--recursive", is_flag=True, help="List assets recursively.")
@click.pass_context
def list_assets(ctx, id, url, object_key, page_size, page_offset, page_offset_token, order_by, order_by_type, recursive):
    """List assets"""
    
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]
    config = ctx.obj["config"]
    
    if not id and not url and not object_key:
        click.echo(json.dumps({"error": "Please provide an id, url, or object-key."}))
        sys.exit(1)

    if page_size and page_size not in range(1, 1000):
        click.echo(json.dumps({ "error": "Page size must be between 1 and 999." }))
        sys.exit(1)
        
    if page_offset and page_offset < 0:
        click.echo(json.dumps({ "error": "Page offset must be greater than or equal to 0." }))
        sys.exit(1)
        
    if page_offset and page_offset_token:
        click.echo(json.dumps({ "error": "Please provide either a page offset or a page offset token." }))
        sys.exit(1)
        
    if page_offset_token:
        try:
            page_offset_token = page_offset_token.replace('\\n', '').replace('\\"', '"')
            page_offset = page_offset_token
        except json.JSONDecodeError:
            click.echo(json.dumps({ "error": "Invalid page offset token." }))
            sys.exit(1)
        
    if order_by_type and (order_by_type == "asc" or order_by_type == "desc"):
        order_by_type += "ending"

    try:        
        filter = None
        if id:
            asset = nomad_sdk.get_asset(id)
            if not asset:
                click.echo(json.dumps({ "error": f"Asset with id {id} not found." }))
                sys.exit(1)        

            filter = [{
                "fieldName": "id",
                "operator": "equals",
                "values": id
            }]
        elif url or object_key:
            if url and "::" not in url:
                click.echo(json.dumps({ "error": "Please provide a valid path or set the default bucket." }))               
                sys.exit(1)
            if object_key:
                if "bucket" in config:
                    url = f"{config["bucket"]}::{object_key}"
                else:
                    click.echo(json.dumps({ "error": "Please set bucket using `set-bucket` or use url." }))
                    sys.exit(1)

            filter = [{
                "fieldName": "url",
                "operator": "equals",
                "values": url
            }]
        else:
            click.echo(json.dumps({ "error": "Please provide an id or path." }))
            sys.exit(1)
            
        sort_field = None
        sort_field = [{
            "fieldName": order_by,
            "sortType": order_by_type
        }]

        asset_properties_results = nomad_sdk.search(None, page_offset, page_size, filter, None,
            None, None, None, None, None, None, None, None, None, None)
        
        asset_properties = asset_properties_results["items"]
        if not asset_properties:
            click.echo(json.dumps({ "error": "No assets found." }))
            sys.exit(1)
            
        results = []
        if asset_properties[0]["identifiers"]["assetTypeDisplay"] == "Folder":
            if recursive:
                filter = [{
                    "fieldName": "uuidSearchField",
                    "operator": "equals",
                    "values": asset_properties[0]["id"]
                }]           
            else:
                filter = [{
                    "fieldName": "parentId",
                    "operator": "equals",
                    "values": asset_properties[0]["id"]
                }]     

            folder_results = nomad_sdk.search(None, page_offset, page_size, filter, sort_field, 
                [
                    { "name": "id"},
                    { "name": "identifiers.name"},
                    { "name": "identifiers.url"},
                    { "name": "identifiers.fullUrl"},
                    { "name": "identifiers.assetTypeDisplay"},
                    { "name": "identifiers.mediaTypeDisplay"},
                    { "name": "identifiers.contentLength"}
                ], None, None, None, None, None, None, None, None, None)
            
            results = folder_results
        else:
            results = nomad_sdk.search(None, page_offset, page_size, filter, sort_field, 
                [
                    { "name": "id"},
                    { "name": "identifiers.name"},
                    { "name": "identifiers.url"},
                    { "name": "identifiers.fullUrl"},
                    { "name": "identifiers.assetTypeDisplay"},
                    { "name": "identifiers.mediaTypeDisplay"},
                    { "name": "identifiers.contentLength"}
                ], None, None, None, None, None, None, None, None, None)

        filtered_result_items = [
            {
                "id": x["id"],
                "lastModifiedDate": x["lastModifiedDate"],
                "createdDate": x["createdDate"],
                "assetTypeDisplay": x["identifiers"]["assetTypeDisplay"],
                "mediaTypeDisplay": x["identifiers"].get("mediaTypeDisplay"),
                "name": x["identifiers"].get("name"),
                "contentLength": x["identifiers"].get("contentLength"),
                "url": x["identifiers"].get("url"),
                "fullUrl": x["identifiers"].get("fullUrl")
            }
            for x in results["items"]
        ]
        
        results["items"] = filtered_result_items

        click.echo(json.dumps(results, indent=4))
    
    except Exception as e:
        click.echo(json.dumps({ "error": f"Error listing assets: {e}" }))
        sys.exit(1)


import click
import json
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--content-id", required=False, help="The content id of the event to update. None for create.")
@click.option("--content-definition-id", required=True, help="The content definition id of the event.")
@click.option("--name", required=False, help="The name of the event.")
@click.option("--start-datetime", required=True, help="The start date time of the event.")
@click.option("--end-datetime", required=True, help="The end date time of the event.")
@click.option("--event-type", required=True, help="The event type of the event in JSON dict format.")
@click.option("--series", required=False, help="The series of the event in JSON dict format.")
@click.option("--is-disabled", required=False, type=click.BOOL, help="Whether the event is disabled.")
@click.option("--override-series-properties", required=True, type=click.BOOL, help="Whether to override the series properties.")
@click.option("--series-properties", required=False, help="The properties of the event in JSON dict format.")
@click.pass_context
def create_and_update_event(ctx, content_id, content_definition_id, name, start_datetime, end_datetime, event_type, series, is_disabled, override_series_properties, series_properties):
    """Create and update event"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        result = nomad_sdk.create_and_update_event(
            content_id,
            content_definition_id,
            name,
            start_datetime,
            end_datetime,
            json.loads(event_type),
            json.loads(series) if series else None,
            is_disabled,
            override_series_properties,
            json.loads(series_properties) if series_properties else None
        )
        click.echo(json.dumps(result, indent=4))

    except Exception as e:
        click.echo(json.dumps({"error": f"Error creating and updating event: {e}"}))
        sys.exit(1)


import click
import json
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--ids", help="The IDs of the assets to be downloaded in JSON list format.")
@click.option("--urls", help="The Nomad URL(s) of the Asset(s) (file or folder) to download the archive(s) for (bucket::object-key).")
@click.option("--object-keys", help="Object-key(s) only of the Asset(s) (file or folder) to download the archive(s) for. This option assumes the default bucket that was previously set with the `set-bucket` command.")
@click.option("--file-name", required=False, help="The file name of the archive asset.")
@click.option("--download-proxy", required=False, type=click.BOOL, help="The download proxy of the archive asset.")
@click.pass_context
def download_archive_asset(ctx, ids, urls, object_keys, file_name, download_proxy):
    """Download archive asset"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    for url in urls:
        if url and "::" not in url:
            click.echo(json.dumps({ "error": "Please provide a valid path or set the default bucket." }))               
            sys.exit(1)
        
        url_search_results = nomad_sdk.search(None, None, None, [{
            "fieldName": "url",
            "operator": "equals",
            "values": url
        }], None, None, None, None, None, None, None, None, None)
        
        if not url_search_results or len(url_search_results["items"] == 0):
            click.echo(json.dumps({ "error": f"URL {url} not found." }))
            sys.exit(1)
            
        ids.append(url_search_results["items"][0]["id"])
        
    for object_key in object_keys:
        if "bucket" in ctx.obj:
            url = f"{ctx.obj['bucket']}::{object_key}"
        else:
            click.echo(json.dumps({ "error": "Please set bucket using `set-bucket` or use url." }))
            sys.exit(1)
        
        url_search_results = nomad_sdk.search(None, None, None, [{
            "fieldName": "url",
            "operator": "equals",
            "values": url
        }], None, None, None, None, None, None, None, None, None)
        
        if not url_search_results or len(url_search_results["items"] == 0):
            click.echo(json.dumps({ "error": f"URL {url} not found." }))
            sys.exit(1)
            
        ids.append(url_search_results["items"][0]["id"])

    try:
        result = nomad_sdk.download_archive_asset(
            ids,
            file_name,
            download_proxy
        )
        click.echo("Archive asset downloaded successfully.")
        click.echo(json.dumps(result, indent=4))

    except Exception as e:
        click.echo(json.dumps({"error": f"Error downloading archive asset: {e}"}))
        sys.exit(1)


from nomad_media_cli.helpers.utils import initialize_sdk
import click
import json
import sys

@click.command()
@click.option("--content-id", required=True, help="The id of the content to retrieve.")
@click.pass_context
def get_content(ctx, content_id):
    """Get content by id"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        result = nomad_sdk.get_content(content_id)
        click.echo(json.dumps(result, indent=4))

    except Exception as e:
        click.echo(json.dumps({ "error": f"Error getting content: {e}" }))
        sys.exit(1)


import click
import json
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--id", required=True, help="The ID of the asset to share.")
@click.option("--nomad-users", required=False, help="The nomad users of the share in JSON list format.")
@click.option("--external-users", required=False, help="The external users of the share in JSON list format.")
@click.option("--shared-duration-in-hours", required=False, type=click.INT, help="The share duration in hours of the share.")
@click.pass_context
def share_asset(ctx, id, nomad_users, external_users, shared_duration_in_hours):
    """Share asset"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        result = nomad_sdk.share_asset(
            id,
            json.loads(nomad_users) if nomad_users else None,
            json.loads(external_users) if external_users else None,
            shared_duration_in_hours
        )
        click.echo("Asset shared successfully.")
        click.echo(json.dumps(result, indent=4))

    except Exception as e:
        click.echo(json.dumps({"error": f"Error sharing asset: {e}"}))
        sys.exit(1)


import click
from click import style

import os
from platformdirs import user_config_dir
import sys

from nomad_media_cli.commands.cli_commands.init import init
from nomad_media_cli.commands.cli_commands.list_config_path import list_config_path
from nomad_media_cli.commands.cli_commands.login import login
from nomad_media_cli.commands.cli_commands.logout import logout
from nomad_media_cli.commands.cli_commands.update_config import update_config

from nomad_media_cli.commands.admin.asset_upload.upload_assets import upload_assets

from nomad_media_cli.commands.common.asset.delete_asset import delete_asset
from nomad_media_cli.commands.common.asset.get_asset_details import get_asset_details
from nomad_media_cli.commands.common.asset.list_assets import list_assets
from nomad_media_cli.commands.common.asset.list_buckets import list_buckets
from nomad_media_cli.commands.common.asset.set_default_bucket import set_default_bucket

from nomad_media_cli.commands.common.content_metadata.add_asset_collection import add_asset_collection
from nomad_media_cli.commands.common.content_metadata.add_asset_properties import add_asset_properties
from nomad_media_cli.commands.common.content_metadata.add_asset_related_content import add_asset_related_content
from nomad_media_cli.commands.common.content_metadata.add_asset_tag import add_asset_tag
from nomad_media_cli.commands.common.content_metadata.list_asset_collections import list_asset_collections
from nomad_media_cli.commands.common.content_metadata.list_asset_related_contents import list_asset_related_contents
from nomad_media_cli.commands.common.content_metadata.list_asset_tags import list_asset_tags
from nomad_media_cli.commands.common.content_metadata.remove_asset_collection import remove_asset_collection
from nomad_media_cli.commands.common.content_metadata.remove_asset_related_content import remove_asset_related_content
from nomad_media_cli.commands.common.content_metadata.remove_asset_tag import remove_asset_tag

from nomad_media_cli.commands.common.search.get_content_definition_contents import get_content_definition_contents

from nomad_media_cli.helpers.check_token import check_token

# Set the configuration directory and path
CONFIG_DIR = user_config_dir("nomad_media_cli")
CONFIG_PATH = os.path.join(CONFIG_DIR, "config.json")

@click.group()
@click.option("--config-path", default=CONFIG_PATH, help="Path to the configuration file (optional)")
@click.pass_context
def cli(ctx, config_path):
    """Nomad Media CLI"""
    ctx.ensure_object(dict)
    ctx.obj["config_path"] = config_path

def add_command(command):
    #@click.command(name=command.name)
    #@click.pass_context
    #def wrapper(ctx, *args, **kwargs):
    #    with click.progressbar(length=100, label=f"Running {command.name}...", file=sys.stderr) as progress:
    #        result = ctx.invoke(command, *args, **kwargs)
    #        progress.update(100)
    #        progress.finish()  
    #    return result       

    return cli.add_command(command)   
    
# CLI_Commands
add_command(init)
add_command(list_config_path)
add_command(login)
add_command(logout)
add_command(update_config)

# Admin
# Asset Upload
add_command(upload_assets)

# Common
# Asset
add_command(delete_asset)
add_command(get_asset_details)
add_command(list_assets)
add_command(list_buckets)
add_command(set_default_bucket)

# Content Metadata
add_command(add_asset_collection)
add_command(add_asset_properties)
add_command(add_asset_related_content)
add_command(add_asset_tag)
add_command(list_asset_collections)
add_command(list_asset_tags)
add_command(list_asset_related_contents)
add_command(remove_asset_collection)
add_command(remove_asset_related_content)
add_command(remove_asset_tag)

# Search
add_command(get_content_definition_contents)

@cli.result_callback()
@click.pass_context
def process(ctx, *args, **kwargs):
    if ctx.obj.get("nomad_sdk"):
        check_token(ctx.obj["config_path"], ctx.obj["nomad_sdk"])

if __name__ == "__main__":
    cli(obj={})



import click
import json
import sys

from nomad_media_cli.helpers.utils import initialize_sdk
from nomad_media_cli.helpers.get_id_from_url import get_id_from_url

@click.command()
@click.option("--id", help="The id of the asset.")
@click.option("--url", help="The Nomad URL of the Asset (file or folder) to remove the collection from (bucket::object-key).")
@click.option("--object-key", help="Object-key of the Asset (file or folder) to remove the collection from. This option assumes the default bucket that was previously set with the `set-bucket` command.")
@click.option("--collection-id", required=True, help="The id of the collection.")
@click.pass_context
def remove_asset_collection(ctx, id, url, object_key, collection_id):
    """Remove collection from asset"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]
    
    if not id and not url and not object_key:
        click.echo(json.dumps({ "error": "Please provide an id, url or object-key" }))
        sys.exit(1)

    if url or object_key:
        id = get_id_from_url(ctx, url, object_key, nomad_sdk)

    try:
        result = nomad_sdk.remove_tag_or_collection("collection", id, "asset", collection_id)
        if not result:
            click.echo(json.dumps({ "error": "Error removing collection from asset." }))
            sys.exit(1)
            
        click.echo(json.dumps({ "message": "Collection removed from asset." }))
    except Exception as e:
        click.echo(json.dumps({ "error": f"Error removing collection from asset: {e}" }))


import click
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--email", required=True, help="The email of the user.")
@click.pass_context
def resend_code(ctx, email):
    """Resend verification code"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        nomad_sdk.resend_code(email)
        click.echo("Verification code resent successfully.")

    except Exception as e:
        click.echo(json.dumps({"error": f"Error resending verification code: {e}"}))
        sys.exit(1)


import click
import json
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--content-management-type", required=False, type=click.INT, help="The type of content management to get. 1; None, 2; DataSelector, 3; FormSelector")
@click.option("--sort-column", required=False, help="The column to sort by.")
@click.option("--is-desc", required=False, type=click.BOOL, help="Whether to sort descending.")
@click.option("--page-index", required=False, type=click.INT, help="The page index to get.")
@click.option("--page-size", required=False, type=click.INT, help="The page size to get.")
@click.pass_context
def get_content_definitions(ctx, content_management_type, sort_column, is_desc, page_index, page_size):
    """Get content definitions"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        if content_management_type:
            if content_management_type < 1 or content_management_type > 3:
                click.echo(json.dumps({"error": "Content management type must be 1, 2, or 3."}))
                sys.exit(1)     

        result = nomad_sdk.get_content_definitions(content_management_type, sort_column, is_desc, page_index, page_size)
        click.echo(json.dumps(result, indent=4))

    except Exception as e:
        click.echo(json.dumps({"error": f"Error getting content definitions: {e}"}))
        sys.exit(1)


import click
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.pass_context
def forgot_password(ctx):
    """Forgot password"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        nomad_sdk.forgot_password()
        click.echo("Forgot password email sent successfully.")

    except Exception as e:
        click.echo(json.dumps({"error": f"Error sending forgot password email: {e}"}))
        sys.exit(1)


import click
import json
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--email", required=True, help="The email of the user.")
@click.option("--first-name", required=False, help="The first name of the user.")
@click.option("--last-name", required=False, help="The last name of the user.")
@click.option("--password", required=True, help="The password of the user.")
@click.pass_context
def register(ctx, email, first_name, last_name, password):
    """Register user"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        result = nomad_sdk.register(email, first_name, last_name, password)
        click.echo("User registered successfully.")
        click.echo(json.dumps(result, indent=4))

    except Exception as e:
        click.echo(json.dumps({"error": f"Error registering user: {e}"}))
        sys.exit(1)


import click
import json
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--sources", required=True, help="The sources of the media in JSON list format.")
@click.option("--title", required=False, help="The title of the media.")
@click.option("--tags", required=False, help="The tags of the media in JSON list format.")
@click.option("--collections", required=False, help="The collections of the media in JSON list format.")
@click.option("--related-contents", required=False, help="The related contents of the media in JSON list format.")
@click.option("--destination-folder-id", required=True, help="The destination folder ID of the media.")
@click.option("--video-bitrate", required=False, type=click.INT, help="The video bitrate of the media.")
@click.option("--audio-tracks", required=False, help="The audio tracks of the media in JSON list format.")
@click.pass_context
def build_media(ctx, sources, title, tags, collections, related_contents, destination_folder_id, video_bitrate, audio_tracks):
    """Build media"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        nomad_sdk.build_media(
            json.loads(sources),
            title,
            json.loads(tags) if tags else None,
            json.loads(collections) if collections else None,
            json.loads(related_contents) if related_contents else None,
            destination_folder_id,
            video_bitrate,
            json.loads(audio_tracks) if audio_tracks else None
        )
        click.echo("Media built successfully.")

    except Exception as e:
        click.echo(json.dumps({"error": f"Error building media: {e}"}))
        sys.exit(1)


import click
import json
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--email", required=True, help="The email of the user.")
@click.option("--code", required=True, help="The code of the user.")
@click.pass_context
def verify(ctx, email, code):
    """Verify user account"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        result = nomad_sdk.verify(email, code)
        click.echo("User account verified successfully.")
        click.echo(json.dumps(result, indent=4))

    except Exception as e:
        click.echo(json.dumps({"error": f"Error verifying user account: {e}"}))
        sys.exit(1)


import click
import json
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--live-channel-id", required=True, help="The ID of the live channel.")
@click.option("--name", required=False, help="The name of the live channel.")
@click.option("--thumbnail-image-id", required=False, help="The thumbnail image ID of the live channel.")
@click.option("--archive-folder-asset-id", required=False, help="The archive folder asset ID of the live channel.")
@click.option("--enable-high-availability", required=False, type=click.BOOL, help="Indicates if the live channel is enabled for high availability.")
@click.option("--enable-live-clipping", required=False, type=click.BOOL, help="Indicates if the live channel is enabled for live clipping.")
@click.option("--is-secure-output", required=True, type=click.BOOL, help="Indicates if the live channel is secure output.")
@click.option("--is-output-screenshot", required=True, type=click.BOOL, help="Indicates if the live channel is output screenshot.")
@click.option("--channel-type", required=False, help="The type of the live channel. The types are External, IVS, Normal, and Realtime.")
@click.option("--external-service-api-url", required=False, help="The external service API URL of the live channel. Only required if the type is External.")
@click.option("--security-groups", required=False, help="The security groups of the live channel in JSON list format.")
@click.pass_context
def update_live_channel(ctx, live_channel_id, name, thumbnail_image_id, archive_folder_asset_id, enable_high_availability, enable_live_clipping, is_secure_output, is_output_screenshot, channel_type, external_service_api_url, security_groups):
    """Update live channel"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        result = nomad_sdk.update_live_channel(
            live_channel_id,
            name,
            thumbnail_image_id,
            archive_folder_asset_id,
            enable_high_availability,
            enable_live_clipping,
            is_secure_output,
            is_output_screenshot,
            channel_type,
            external_service_api_url,
            json.loads(security_groups) if security_groups else None
        )
        click.echo(json.dumps(result, indent=4))

    except Exception as e:
        click.echo(json.dumps({"error": f"Error updating live channel: {e}"}))
    sys.exit(1)


import click
import json
import sys
from nomad_media_cli.helpers.utils import initialize_sdk

@click.command()
@click.option("--id", required=False, help="The ID of the asset to register.")
@click.option("--parent-id", required=False, help="The ID of the parent.")
@click.option("--display-object-key", required=False, help="The display object key of the register.")
@click.option("--bucket-name", required=True, help="The bucket name of the register.")
@click.option("--object-key", required=True, help="The object key of the register.")
@click.option("--e-tag", required=False, help="The eTag of the register.")
@click.option("--tag-ids", required=False, help="The tags of the register in JSON list format.")
@click.option("--collection-ids", required=False, help="The collections of the register in JSON list format.")
@click.option("--related-content-ids", required=False, help="The related contents of the register in JSON list format.")
@click.option("--sequencer", required=False, help="The sequencer of the register.")
@click.option("--asset-status", required=False, help="The asset status of the register.")
@click.option("--storage-class", required=False, help="The storage class of the register.")
@click.option("--asset-type", required=False, help="The asset type of the register.")
@click.option("--content-length", required=False, type=click.INT, help="The content length of the register.")
@click.option("--storage-event-name", required=False, help="The storage event name of the register.")
@click.option("--created-date", required=False, help="The created date of the register.")
@click.option("--storage-source-ip-address", required=False, help="The storage source IP address of the register.")
@click.option("--start-media-processor", required=False, type=click.BOOL, help="The start media processor of the register.")
@click.option("--delete-missing-asset", required=False, type=click.BOOL, help="The delete missing asset of the register.")
@click.pass_context
def register_asset(ctx, id, parent_id, display_object_key, bucket_name, object_key, e_tag, tag_ids, collection_ids, related_content_ids, sequencer, asset_status, storage_class, asset_type, content_length, storage_event_name, created_date, storage_source_ip_address, start_media_processor, delete_missing_asset):
    """Register asset"""
    initialize_sdk(ctx)
    nomad_sdk = ctx.obj["nomad_sdk"]

    try:
        result = nomad_sdk.register_asset(
            id,
            parent_id,
            display_object_key,
            bucket_name,
            object_key,
            e_tag,
            json.loads(tag_ids) if tag_ids else None,
            json.loads(collection_ids) if collection_ids else None,
            json.loads(related_content_ids) if related_content_ids else None,
            sequencer,
            asset_status,
            storage_class,
            asset_type,
            content_length,
            storage_event_name,
            created_date,
            storage_source_ip_address,
            start_media_processor,
            delete_missing_asset
        )
        click.echo("Asset registered successfully.")
        click.echo(json.dumps(result, indent=4))

    except Exception as e:
        click.echo(json.dumps({"error": f"Error registering asset: {e}"}))
        sys.exit(1)